'** QDVESA.BAS
'** QBasic source
'** May, 1997
'** DESCRIPTION: Routines to emulate simple QBasic graphics
'** commands in 256-color SVGA modes.  You must have VESA VBE
'** capability, either through a software driver or 'built-in' to
'** your video card (most newer cards have this).  Routines here
'** may still fail to operate on all cards.  QuickBasic users
'** should load this with the /lqb switch.
'**
'** Stephen L. Maxson
'** smax@isc-durant.com
'**
'** CREDITS:
'** InterruptX2 procedure: Copyright (c) 1992, by Hans Lunsing.
'** Routines in the VLINE, VCIRCLE, and VTEXT and VTLINE SUBs are
'**   from code by Douglas H. Lusher.
'** BIOS character printing routine in the VPRINT SUB is by
'**   D. Tyler Barnes.
'** Routine in the VPAINT SUB is from "BLOKFILL.BAS", Copyright
'**   (c) 1996, by Daniel J. Arigan.
'** The FillChar SUB (used by VCLS) is by Jonathan Leger.
'** VESA ModeInfoBlock structure "borrowed" from Nick Coons, who
'**   also provided other valuable information.
'**
'** Public Domain, (ab)use at your own risk
'**
DECLARE FUNCTION GetKey$ ()
DECLARE SUB TimeDelay (Time!, Delay!)
DECLARE SUB RotatePalette (BeginColour%, EndColour%)
'** Next five routines are required for VESA VBE calls.
DECLARE FUNCTION VesaStatus% ()
DECLARE SUB GetVESAInfo (Mode%)
DECLARE SUB SetVESAMode (Mode%)
DECLARE SUB SwitchBank (Win%)
DECLARE SUB InterruptX2 (IntNo%, Inreg AS ANY, OutReg AS ANY)
'** Routine below required by the VCLS SUB.
DECLARE SUB FillChar (segment%, offset%, value%, bytes%)

'** Below are the "emulated" commands

DECLARE SUB VCIRCLE (X%, Y%, Radius%, Colour%)
'** Usage: VCIRCLE x%, y%, radius%, color%
'** Draws a circle at x%, y%.

DECLARE SUB VCLS (Colour%)
'** Usage: VCLS color%
'** Clears the current screen mode with color%.

DECLARE SUB VLINE (X1%, Y1%, X2%, Y2%, Colour%)
'** Usage: VLINE x1%, y1%, x2%, y2%, color%
'** Draws a line from x1%, y1% to x2%, y2%.

DECLARE SUB VLINEBOX (X1%, Y1%, X2%, Y2%, Colour%, B$)
'** Usage: VLINEBOX x1%, y1%, x2%, y2%, color%, {"B" | "BF"}
'** "B" draws a rectangle using X1%, Y1% and X2%, Y2% as opposing
'**   corners.
'** "BF" draws a filled rectangle.

DECLARE SUB VPAINT (X%, Y%, Colour%, Border%)
'** Usage: VPAINT x%, y%, fillcolor%, {bordercolor% | -1}
'** Fills an area begining at x%, y% with fillcolor%.
'** Stops when it reaches bordercolor% or another instance of
'**   fillcolor%.
'** If bordercolor% is specified as -1, painting occurs only on
'**   pixels already painted with the color which is at x%, y% when
'**   the routine is called, any other color stops the fill.

DECLARE FUNCTION VPOINT% (X%, Y%)
'** Usage: VPOINT(x%, y%)
'** Returns color value at x%, y%.

DECLARE SUB VPRINT (Row%, Column%, Text$, Colour%)
'** Usage: VPRINT row%, column%, text$, color%
'** Prints text at specified row and column, wraparound to column 1
'**   is handled.  This is an actual BIOS call, and is far superior
'**   in speed to VPRINT2 or VTEXT.  However, it cannot use
'**   "compressed" characters or print at pixel locations.

DECLARE SUB VPRINT2 (Row%, Column%, Text$, Colour%, Lines%)
'** Usage: VPRINT2 row%, column%, text$, color%, {1 | 2}
'** Prints text at specified row and column, wraparound to column 1
'**   is handled.
'** If Lines% = 1, 8x16 characters are generated.
'** If Lines% = 2, 8x8 characters are generated.

DECLARE SUB VPSET (X%, Y%, Colour%)
'** Usage: VPSET x%, y%, color%
'** Plots a single point at x%, y%.

DECLARE SUB VTEXT (X%, Y%, Text$, Colour%, Lines%)
'** Usage: VTEXT x%, y%, text$, color%, {1 | 2}
'** Prints text at specified x%, y% pixel location.
'** If Lines% = 1, 8x16 characters are generated.
'** If Lines% = 2, 8x8 characters are generated.

DECLARE SUB VTLINE (X%, Y%, Colour%, LineFormat%)
'** Usage: Used by the VTEXT SUB to speed character printing.

DEFINT A-Z

'** VESA VbeInfoBlock structure.
TYPE VbeInfoBlock
  VbeSignature          AS STRING * 4
  VbeVersion            AS INTEGER
  OemStringPtr          AS LONG
  Capabilities          AS STRING * 4
  VideoModePtr          AS LONG
  TotalMemory           AS INTEGER
  '** From here down are VESA VBE 2.0 additions.
  OemSoftwareRev        AS INTEGER
  OemVendorNamePtr      AS LONG
  OemProductNamePtr     AS LONG
  OemProductRevPtr      AS LONG
  Reserved              AS STRING * 222
  OemData               AS STRING * 256
END TYPE

'** VESA ModeInfoBlock structure.
TYPE ModeInfoBlock
  Attr                  AS INTEGER
  WinAAttr              AS STRING * 1
  WinBAttr              AS STRING * 1
  Gran                  AS INTEGER
  Size                  AS INTEGER
  WinASeg               AS INTEGER
  WinBSeg               AS INTEGER
  FarWindow             AS LONG
  BytesScanLine         AS INTEGER
  WidthPixels           AS INTEGER
  HeightPixels          AS INTEGER
  WidthChar             AS STRING * 1
  HeightChar            AS STRING * 1
  Planes                AS STRING * 1
  BitsPerPixel          AS STRING * 1
  Banks                 AS STRING * 1
  MemoryModel           AS STRING * 1
  BankSize              AS STRING * 1
  Pages                 AS STRING * 1
  Reserved              AS STRING * 1
  RedMask               AS STRING * 1
  RedField              AS STRING * 1
  GreenMask             AS STRING * 1
  GreenField            AS STRING * 1
  BlueMask              AS STRING * 1
  BlueField             AS STRING * 1
  ReservedMask          AS STRING * 1
  ReservedPosition      AS STRING * 1
  DirectColorInfo       AS STRING * 1
  VideoBuffer           AS LONG
  OffscreenMemory       AS LONG
  KBOffscreenMemory     AS INTEGER
  Reserved2             AS STRING * 206
END TYPE

'** Register type for use with InterruptX.
TYPE RegTypeX
  AX    AS INTEGER
  BX    AS INTEGER
  CX    AS INTEGER
  DX    AS INTEGER
  BP    AS INTEGER
  SI    AS INTEGER
  DI    AS INTEGER
  Flags AS INTEGER
  DS    AS INTEGER
  ES    AS INTEGER
END TYPE

CONST FALSE = 0, TRUE = NOT FALSE

DIM VbeInfoBlock AS VbeInfoBlock, ModeInfoBlock AS ModeInfoBlock
DIM Register AS RegTypeX
DIM VESAMode%(4)

VESAMode%(1) = &H100                     ' 640x400x256
VESAMode%(2) = &H101                     ' 640x480x256
VESAMode%(3) = &H103                     ' 800x600x256
VESAMode%(4) = &H105                     '1024x768x256

SCREEN 0, 0, 0
WIDTH 80, 25
CLS

IF VesaStatus% = 0 THEN
  PRINT "VESA VGA Bios Extensions do not appear to be available."
  PRINT "Install the appropriate VESA VBE driver for your video card,"
  PRINT "and try again."
  END
END IF

DO
  DO
    PRINT
    PRINT "(1)  -   640x400x256"
    PRINT "(2)  -   640x480x256"
    PRINT "(3)  -   800x600x256"
    PRINT "(4)  -  1024x768x256"
    PRINT
    INPUT "Display in which mode? (1-4, 0 to exit) >", A$
    VM% = VAL(A$)
  LOOP UNTIL VM% > -1 AND VM% < 5
  IF VM% = 0 THEN END
  '** Get the info on the selected mode.
  GetVESAInfo VM%
  '** Is the mode available?
  IF (ModeInfoBlock.Attr AND 1) THEN EXIT DO
  PRINT "That mode does not seem to be supported.  Please select again."
LOOP

'** Set 256 color mode with SCREEN 13 (320x200x256).
'** Necessary for using 256 color PALETTE statements.
SCREEN 13
SetVESAMode VM%

'** Create vertical reference table.
DIM RefTable&(ModeInfoBlock.HeightPixels - 1)
FOR I& = 0 TO ModeInfoBlock.HeightPixels - 1
  RefTable&(I&) = I& * ModeInfoBlock.BytesScanLine
NEXT I&

'** How many banks (windows) do we have?
WinSize& = ModeInfoBlock.Size * 1024&
Temp& = ModeInfoBlock.HeightPixels
Temp& = Temp& * ModeInfoBlock.BytesScanLine
IF Temp& MOD WinSize& <> 0 THEN NumWindows% = 1
NumWindows% = NumWindows% + Temp& \ WinSize&

'** These just save some typing and math calculations in the demos.
Mheight% = ModeInfoBlock.HeightPixels
Mwidth% = ModeInfoBlock.WidthPixels
Sheight% = Mheight% - 1
Swidth% = Mwidth% - 1

'** A small starfield scroll to demo VPSET.
RANDOMIZE TIMER
NS% = 31
DIM S1X%(NS%), S1Y%(NS%), S2X%(NS%), S2Y%(NS%), S3X%(NS%), S3Y%(NS%)
FOR I% = 0 TO NS%
  S1X%(I%) = INT(RND * Mwidth%)
  S1Y%(I%) = INT(RND * Mheight%)
  S2X%(I%) = INT(RND * Mwidth%)
  S2Y%(I%) = INT(RND * Mheight%)
  S3X%(I%) = INT(RND * Mwidth%)
  S3Y%(I%) = INT(RND * Mheight%)
NEXT I%
DO
  FOR I% = 0 TO NS%
    VPSET S1X%(I%), S1Y%(I%), 0
    S1Y%(I%) = S1Y%(I%) + 8
    IF S1Y%(I%) > Sheight% THEN S1Y%(I%) = S1Y%(I%) - Sheight%
    VPSET S1X%(I%), S1Y%(I%), 15
    VPSET S2X%(I%), S2Y%(I%), 0
    S2Y%(I%) = S2Y%(I%) + 4
    IF S2Y%(I%) > Sheight% THEN S2Y%(I%) = S2Y%(I%) - Sheight%
    VPSET S2X%(I%), S2Y%(I%), 7
    VPSET S3X%(I%), S3Y%(I%), 0
    S3Y%(I%) = S3Y%(I%) + 2
    IF S3Y%(I%) > Sheight% THEN S3Y%(I%) = S3Y%(I%) - Sheight%
    VPSET S3X%(I%), S3Y%(I%), 8
  NEXT I%
LOOP WHILE INKEY$ = ""
ERASE S1X%, S1Y%, S2X%, S2Y%, S3X%, S3Y%

'** Clear the screen.  Note that setting the mode also dumps video
'** memory, so a call to SetVESAMode would also blank the screen,
'** although it causes the screen to 'twitch' slightly.  The VCLS
'** SUB appears to be just as fast, doesn't cause the 'twitch', and
'** also allows you to clear the screen to any color you like.
VCLS 0
'** Draw a moire pattern to demo VLINE.
C% = 31
FOR I% = 0 TO Sheight% STEP 4
  C% = C% + 1
  IF C% > 72 THEN C% = 32
  VLINE 0, 0, Swidth%, Sheight% - I%, C%
  VLINE Swidth%, Sheight%, 0, I%, C%
  VLINE 0, Sheight%, Swidth%, I%, C%
  VLINE Swidth%, 0, 0, Sheight% - I%, C%
  IF INKEY$ <> "" THEN EXIT FOR
NEXT I%
'** Never hurts to be flashy in a demo.
DO
  Time! = TIMER
  RotatePalette 32, 72
  TimeDelay Time!, .05
LOOP WHILE INKEY$ = ""
PALETTE
BEEP
A$ = GetKey$

VCLS 0
'** Draw box and filled box to demo VLINEBOX.
VLINEBOX 10, 10, 500, 320, 12, "B"
VLINEBOX 100, 100, 160, 130, 10, "BF"
'** Let's try printing to the screen to demo VPRINT2.
VPRINT2 10, 10, "Whoosh, text generation is now *much* faster!", 5, 1
VPRINT2 25, 25, "Very", 9, 2
VPRINT2 26, 25, "quick", 11, 2
VPRINT2 27, 24, "indeed!", 13, 2
VPRINT2 28, 21, "(considering)", 14, 2
T$ = "Here's a rather long line.  This should test the line-wrap "
T$ = T$ + "feature of VPRINT2.  Try it in different modes!  Impress "
T$ = T$ + "your friends!  Win a free toaster with every 17.3 new "
T$ = T$ + "members you sign up!  I'm serious!  The call is free!  "
T$ = T$ + "STaRt tOdAy!!!"
VPRINT2 23, 1, T$, 15, 1
BEEP
A$ = GetKey$

VCLS 1
'** Demo the speed of VPRINT as compared to VPRINT2.
VPRINT2 1, 1, "VPRINT2:", 14, 1
T$ = "This screen is to show you the difference between the VPRINT and "
T$ = T$ + "VPRINT2 routines.  VPRINT2 is slower, and shows an occasional "
T$ = T$ + "'bug' whenever a character falls across the very end of a bank "
T$ = T$ + "(the last 1536 bytes).  Each bank is 65536 bytes in size, but "
T$ = T$ + "screen 13 only uses 320 x 200 = 64000 bytes.  So, the last "
T$ = T$ + "1536 bytes are inaccessible through use of the regular LINE "
T$ = T$ + "command, and as a result, some of the characters will appear "
T$ = T$ + "to be missing scanlines.  The real use for the VPRINT2 and "
T$ = T$ + "VTEXT routines are in cases where you wish to use 'compressed' "
T$ = T$ + "characters (double the number of rows), or in cases where you "
T$ = T$ + "wish to write characters at a specific pixel location.  "
T$ = T$ + "VPRINT2 and VTEXT also write characters transparently, "
T$ = T$ + "preserving the background.  For most applications, however, "
T$ = T$ + "you will probably want to stick with VPRINT."
VPRINT2 3, 1, T$, 10, 2
BEEP
A$ = GetKey$
VPRINT 12, 1, "VPRINT:", 14
VPRINT 13, 1, T$, 10
BEEP
A$ = GetKey$

VCLS 0
'** Draw random circles to demo VCIRCLE.
'** Then fill them to demo VPAINT.
DO
  R% = INT(RND * 50) + 10
  X% = INT(RND * (Mwidth% - R% * 2)) + R%
  Y% = INT(RND * (Mheight% - R% * 2)) + R%
  C% = INT(RND * 40) + 32
  VCIRCLE X%, Y%, R%, C%
  VPAINT X%, Y%, INT(RND * 40) + 32, C%
LOOP WHILE INKEY$ = ""

VCLS 35
'** This demos VTEXT, and gives a better demo of VPAINT.
VLINEBOX 50, 50, 255, 105, 14, "B"
VLINEBOX 300, 50, 410, 260, 9, "B"
A$ = "This is the last screen."
FOR I% = 1 TO LEN(A$)
  VTEXT 50 + I% * 8, 60 + I%, MID$(A$, I%, 1), 14, 1
  VTEXT 300 + I% * 4, 50 + I% * 8, MID$(A$, I%, 1), 12, 2
NEXT I%
VCIRCLE 65, 90, 10, 14
VPAINT 65, 90, 10, 14
VLINEBOX 80, 85, 105, 100, 12, "B"
VLINEBOX 200, 55, 250, 60, 11, "BF"
VCIRCLE 315, 240, 10, 10
VPAINT 315, 240, 8, 10
VCIRCLE 315, 240, 7, 11
'** Demo VPAINT with border color.
VPAINT 51, 51, 12, 14
'** Demo VPAINT without border color.
VPAINT 301, 51, 14, -1
VPAINT 315, 240, 9, -1
BEEP
A$ = GetKey$

SCREEN 0, 0, 0
WIDTH 80
END

'-----------------------------------------------------------------------
' The machine code of the interrupt procedure for InterruptX2
'-----------------------------------------------------------------------
InterruptXASM:
' Number of bytes
DATA  190
' Hexadecimal representation of machine code
DATA 55,8B,EC,8B,5E,0C,8B,17,0A,F6
DATA 74,07,C7,07,FF,FF,E9,A7,00,8B
DATA 5E,06,8B,1F,2E,88,97,77,00,32
DATA C0,80,FA,25,74,05,80,FA,26,75
DATA 02,0C,02,50,1E,06,56,57,9C,8B
DATA 76,0A,80,FA,20,7C,05,80,FA,30
DATA 7C,0A,81,7C,08,FF,FF,74,03,8B
DATA 6C,08,8B,44,0E,25,D5,0F,50,8B
DATA 04,8B,5C,02,8B,4C,04,8B,54,06
DATA 8B,7C,0C,FF,74,0A,81,7C,12,FF
DATA FF,74,03,8E,44,12,81,7C,10,FF
DATA FF,74,03,8E,5C,10,5E,9D,CD,00
DATA 55,8B,EC,9C,83,C5,0E,F6,46,FE
DATA 02,74,02,45,45,1E,56,8E,5E,FC
DATA 8B,76,08,89,04,89,5C,02,89,4C
DATA 04,89,54,06,8F,44,0A,89,7C,0C
DATA 8F,44,10,8C,44,12,8F,44,0E,8F
DATA 44,08,F6,46,FE,02,74,02,44,44
DATA 9D,5F,5E,07,1F,58,5D,CA,08,00

'*****************************************************
'***                 FillChar()                    ***
'*****************************************************
'***  FillChar() puts whatever is in value% (which ***
'***  should be a number from 0-255) into memory   ***
'***  starting at location segment:offset, ending  ***
'***  at location segment:offset+bytes%.  An good  ***
'***  example of its use would be for clearing the ***
'***  screen with a different background color in  ***
'***  a graphics screen (which is pitifully slow   ***
'***  usint PSET).  To do this for SCREEN 13, for  ***
'***  example:                                     ***
'***                                               ***
'***  FillChar &HA000, 0, 15, &HFA00               ***
'***             ^     ^   ^    ^                  ***
'***             |     |   |    |                  ***
'***  Screen 13--+     |   |    |                  ***
'***                   |   |    |                  ***
'*** Start with first  |   |    |                  ***
'*** pixel.------------+   |    |                  ***
'***                       |    |                  ***
'*** Fill with character/  |    |                  ***
'*** color 15--------------+    |                  ***
'***                            |                  ***
'*** Do so 64,000 times---------+                  ***
'***                                               ***
'*** This will "clear" SCREEN 13 with the color 15 ***
'*** (bright white), and it does so _faster_ than  ***
'*** the CLS routine clears SCREEN 13 in black.    ***
'***                                               ***
'*** Notice that the 64,000 is in HEX (FA00).  This***
'*** is the same as with MemCopy(), where a value  ***
'*** greater than 32,767 has to be put into hex.   ***
'*** Since BASIC integers are signed (can be plus  ***
'*** or minues 32,767), BASIC does not let you use ***
'*** 65,534 (64k) in an integer, and there is no   ***
'*** way to declare a variable as an unsigned int- ***
'*** eger.  Machine Language, however, does not    ***
'*** recognize the plus or minus of a number unless***
'*** you tell it to, so by using the HEX value, we ***
'*** can trick BASIC into passing a number larger  ***
'*** than 32767 to the Machine Language routine,   ***
'*** which will treat &HFA00 as 64000 (even though ***
'*** if you do a PRINT &HFA00 it returns -1536).   ***
'*****************************************************
SUB FillChar (segment%, offset%, value%, bytes%)

asm$ = ""
asm$ = asm$ + CHR$(85)                             'PUSH BP
asm$ = asm$ + CHR$(137) + CHR$(229)                'MOV BP,SP
asm$ = asm$ + CHR$(139) + CHR$(78) + CHR$(6)       'MOV CX,[BP+06]
asm$ = asm$ + CHR$(139) + CHR$(86) + CHR$(8)       'MOV DX,[BP+08]
asm$ = asm$ + CHR$(139) + CHR$(70) + CHR$(12)      'MOV AX,[BP+0C]
asm$ = asm$ + CHR$(30)                             'PUSH DS
asm$ = asm$ + CHR$(142) + CHR$(216)                'MOV DS,AX
asm$ = asm$ + CHR$(139) + CHR$(94) + CHR$(10)      'MOV BX,[BP+0A]
asm$ = asm$ + CHR$(136) + CHR$(23)                 'MOV [BX],DL <------+
asm$ = asm$ + CHR$(67)                             'INC BX             |
asm$ = asm$ + CHR$(226) + CHR$(251)                'LOOP 0112   -------+
asm$ = asm$ + CHR$(31)                             'POP DS
asm$ = asm$ + CHR$(93)                             'POP BP
asm$ = asm$ + CHR$(203)                            'RETF

DEF SEG = VARSEG(asm$)
   CALL ABSOLUTE(BYVAL segment%, BYVAL offset%, BYVAL value%, BYVAL bytes%, SADD(asm$))
DEF SEG

END SUB

FUNCTION GetKey$

DO
  A$ = INKEY$
LOOP WHILE A$ = ""
GetKey$ = A$

END FUNCTION

SUB GetVESAInfo (Mode%)
SHARED VESAMode%(), Register AS RegTypeX, ModeInfoBlock AS ModeInfoBlock

Register.AX = &H4F01                    '4F01H - Get VESA Mode Information
Register.CX = VESAMode%(Mode%)           'Selected Mode
Register.ES = VARSEG(ModeInfoBlock)     'Where InfoBlock is to be stored
Register.DI = VARPTR(ModeInfoBlock)     '  "       "      "  "  "   "
InterruptX2 &H10, Register, Register

END SUB

SUB InterruptX2 (IntNo%, Inreg AS RegTypeX, OutReg AS RegTypeX) STATIC

'-----------------------------------------------------------------------
' InterruptX2
' Interrupt procedure for QBASIC
' (c) 1992 Hans Lunsing
' ----------------------------------------------------------------------

'-----------------------------------------------------------------------
' Interrupt procedure. Works in the same way as its QB 4.5 counterpart.
'-----------------------------------------------------------------------

IF NOT MachineCode% THEN
  ' First time dimension string array to hold machine code
  RESTORE InterruptXASM
  READ nASMBYTES%
  REDIM ASMBuffer(0 TO nASMBYTES% - 1) AS STRING * 1
END IF

' Get address of machine code
DEF SEG = VARSEG(ASMBuffer(0))
offset% = VARPTR(ASMBuffer(0))

IF NOT MachineCode% THEN
  ' First time load string array with machine code
  FOR I% = 0 TO nASMBYTES% - 1
    READ code$
    POKE offset% + I%, VAL("&H" + code$)
  NEXT I%
  ' Indicate availability of machine code
  MachineCode% = TRUE
END IF

' Call interrupt.
' The first Offset% parameter is used by the machine code modifying itself,
' so don't leave it out.
CALL ABSOLUTE(IntNo%, Inreg, OutReg, offset%, offset%)

DEF SEG

END SUB

SUB RotatePalette (BeginColour%, EndColour%)

IF BeginColour% > EndColour% THEN
  Cstep% = -1
ELSEIF BeginColour% < EndColour% THEN
  Cstep% = 1
ELSE
  EXIT SUB
END IF
OUT &H3C7, EndColour%
RedSave% = INP(&H3C9)
GreenSave% = INP(&H3C9)
BlueSave% = INP(&H3C9)
FOR I% = BeginColour% TO EndColour% STEP Cstep%
  OUT &H3C7, I%
  Red% = INP(&H3C9)
  Green% = INP(&H3C9)
  Blue% = INP(&H3C9)
  OUT &H3C8, I%
  OUT &H3C9, RedSave%
  OUT &H3C9, GreenSave%
  OUT &H3C9, BlueSave%
  RedSave% = Red%
  GreenSave% = Green%
  BlueSave% = Blue%
NEXT I%

END SUB

SUB SetVESAMode (Mode%)
SHARED VESAMode%(), CurrentBank%, Register AS RegTypeX

Register.AX = &H4F02                    '4F02H - Set VESA Graphic Mode
Register.BX = VESAMode%(Mode%)           'Selected Mode
InterruptX2 &H10, Register, Register
CurrentBank% = -1

END SUB

SUB SwitchBank (Win%)
SHARED Register AS RegTypeX
       
Register.AX = &H4F05                    '4F05H - Switch Video Bank
Register.BX = &H0                       '0H,0H - Set,Bank A
Register.DX = Win%                      '????H - Appropriate Window
InterruptX2 &H10, Register, Register

END SUB

SUB TimeDelay (Time!, Delay!)

DO
  TimeNow! = TIMER
LOOP UNTIL Time! + Delay! < TimeNow! OR TimeNow! < Time!

END SUB

SUB VCIRCLE (X%, Y%, Radius%, Colour%)
'a routine to draw circles using only integers and integer math
'an implementation of Bresenham's algorithm
'by Douglas H. Lusher, 05-09-1996

A% = 0
B% = Radius%
D% = (1 - Radius%) * 2
XX1% = X%: YY1% = Y% + B%
XX2% = X%: YY2% = Y% - B%
XY1% = X% + B%: YX1% = Y%
XY2% = X% - B%: YX2% = Y%
DO WHILE B% >= A%
  VPSET XX1%, YY1%, Colour%
  VPSET XX1%, YY2%, Colour%
  VPSET XX2%, YY1%, Colour%
  VPSET XX2%, YY2%, Colour%
  VPSET XY1%, YX1%, Colour%
  VPSET XY1%, YX2%, Colour%
  VPSET XY2%, YX1%, Colour%
  VPSET XY2%, YX2%, Colour%
  IF D% + B% > 0 THEN
    B% = B% - 1
    D% = D% - (B% * 2) + 1
    YY1% = YY1% - 1: YY2% = YY2% + 1
    XY1% = XY1% - 1: XY2% = XY2% + 1
  END IF
  IF A% > D% THEN
    A% = A% + 1
    D% = D% + (A% * 2) + 1
    XX1% = XX1% + 1: XX2% = XX2% - 1
    YX1% = YX1% + 1: YX2% = YX2% - 1
  END IF
LOOP

END SUB

SUB VCLS (Colour%)
SHARED NumWindows%, CurrentBank%

'** Keep in mind that this is based on a 64 KB bank size, but I've
'** not yet seen or heard of a card that did not use that size for
'** its banks.  It makes sense to use that size, since it allows for
'** the usage of the 64 KB of main video memory as a page frame.
'** However, it is possible that a VESA VBE implementation might
'** exist that uses a different bank size, in which case this
'** routine will fail, and you will have to resort to setting the
'** mode in order to clear the screen.  The other routines will
'** function normally, with the exception of text printing, which
'** is also based on a 64 KB bank size.
FOR I% = 0 TO NumWindows% - 1
  SwitchBank I%
  LINE (0, 0)-(319, 199), Colour%, BF
  '** The command above clears 64000 bytes, but our banks have an
  '** additional 1536 bytes that need to be cleared.
  FillChar &HA000, &HFA00, Colour%, 1536
NEXT I%
CurrentBank% = -1

END SUB

FUNCTION VesaStatus%
SHARED Register AS RegTypeX, VbeInfoBlock AS VbeInfoBlock

VbeInfoBlock.VbeSignature = "VBE2"      'Return 2.0 info, if available
Register.AX = &H4F00                    '4F00H - Get VESA VBE Information
Register.ES = VARSEG(VbeInfoBlock)      'Where InfoBlock is to be stored
Register.DI = VARPTR(VbeInfoBlock)      '  "       "      "  "  "   "
InterruptX2 &H10, Register, Register
IF Register.AX = &H4F AND VbeInfoBlock.VbeSignature = "VESA" THEN
  VesaStatus% = -1    'QBasic TRUE
ELSE
  VesaStatus% = 0     'QBasic FALSE
END IF

END FUNCTION

SUB VLINE (X1%, Y1%, X2%, Y2%, Colour%)
'a routine to draw lines using only integers and integer math
'an implementation of Bresenham's algorithm
'by Douglas H. Lusher, 05-08-1996

DX2% = 1
A% = X2% - X1%: IF A% < 0 THEN A% = -A%: DX2% = -1
DY2% = 1
B% = Y2% - Y1%: IF B% < 0 THEN B% = -B%: DY2% = -1
DX1% = DX2%: DY1% = 0
IF A% < B% THEN SWAP A%, B%: DX1% = 0: DY1% = DY2%
I1% = B% * 2
D% = I1% - A%
I2% = D% - A%
X% = X1%: Y% = Y1%
FOR I% = 0 TO A%
  VPSET X%, Y%, Colour%
  IF D% < 0 THEN
    X% = X% + DX1%
    Y% = Y% + DY1%
    D% = D% + I1%
  ELSE
    X% = X% + DX2%
    Y% = Y% + DY2%
    D% = D% + I2%
  END IF
NEXT I%

END SUB

SUB VLINEBOX (X1%, Y1%, X2%, Y2%, Colour%, B$)

SELECT CASE B$
  CASE "BF"
    S% = SGN(X2% - X1%)
    IF S% = 0 THEN S% = 1
    FOR I% = X1% TO X2% STEP S%
      VLINE I%, Y1%, I%, Y2%, Colour%
    NEXT I%
  CASE ELSE
    VLINE X1%, Y1%, X1%, Y2%, Colour%
    VLINE X1%, Y1%, X2%, Y1%, Colour%
    VLINE X2%, Y2%, X1%, Y2%, Colour%
    VLINE X2%, Y2%, X2%, Y1%, Colour%
END SELECT

END SUB

SUB VPAINT (X%, Y%, Colour%, Border%)
'** Daniel J. Arigan's "BLOKFILL" routine, modified slightly to
'**   also allow painting on a single background color.
DIM XS%(2047), YS%(2047)
SHARED ModeInfoBlock AS ModeInfoBlock
 
XS%(0) = X%: YS%(0) = Y%: Stk% = 1
IF Border% = -1 THEN PColour% = VPOINT(X%, Y%)
DO WHILE Stk% > 0
  Stk% = Stk% - 1
  X1% = XS%(Stk%): FX1% = 0: Y1% = YS%(Stk%): FY1% = 0
  X2% = XS%(Stk%): FX2% = 0: Y2% = YS%(Stk%): FY2% = 0
  DO WHILE (FX1% + FY1% + FX2% + FY2%) <> 4
    IF FX1% = 0 THEN
      IF X1% = 0 THEN
        FX1% = 1
      ELSE
        X1% = X1% - 1
        FOR I% = Y1% TO Y2%
          NC% = VPOINT(X1%, I%)
          IF Border% = -1 THEN
            IF NC% <> PColour% THEN
              FX1% = 1: X1% = X1% + 1
              EXIT FOR
            END IF
          ELSEIF NC% = Colour% OR NC% = Border% THEN
            FX1% = 1: X1% = X1% + 1
            EXIT FOR
          END IF
        NEXT I%
      END IF
    END IF
    IF FX2% = 0 THEN
      IF X2% = ModeInfoBlock.WidthPixels - 1 THEN
        FX2% = 1
      ELSE
        X2% = X2% + 1
        FOR I% = Y1% TO Y2%
          NC% = VPOINT(X2%, I%)
          IF Border% = -1 THEN
            IF NC% <> PColour% THEN
              FX2% = 1: X2% = X2% - 1
              EXIT FOR
            END IF
          ELSEIF NC% = Colour% OR NC% = Border% THEN
            FX2% = 1: X2% = X2% - 1
            EXIT FOR
          END IF
        NEXT I%
      END IF
    END IF
    IF FY1% = 0 THEN
      IF Y1% = 0 THEN
        FY1% = 1
      ELSE
        Y1% = Y1% - 1
        FOR I% = X1% TO X2%
          NC% = VPOINT(I%, Y1%)
          IF Border% = -1 THEN
            IF NC% <> PColour% THEN
              FY1% = 1: Y1% = Y1% + 1
              EXIT FOR
            END IF
          ELSEIF NC% = Colour% OR NC% = Border% THEN
            FY1% = 1: Y1% = Y1% + 1
            EXIT FOR
          END IF
        NEXT I%
      END IF
    END IF
    IF FY2% = 0 THEN
      IF Y2% = ModeInfoBlock.HeightPixels - 1 THEN
        FY2% = 1
      ELSE
        Y2% = Y2% + 1
        FOR I% = X1% TO X2%
          NC% = VPOINT(I%, Y2%)
          IF Border% = -1 THEN
            IF NC% <> PColour% THEN
              FY2% = 1: Y2% = Y2% - 1
              EXIT FOR
            END IF
          ELSEIF NC% = Colour% OR NC% = Border% THEN
            FY2% = 1: Y2% = Y2% - 1
            EXIT FOR
          END IF
        NEXT I%
      END IF
    END IF
  LOOP
  VLINEBOX X1%, Y1%, X2%, Y2%, Colour%, "BF"
  M1% = -1: M2% = -1
  FOR I% = X1% TO X2%
    DO WHILE Stk% >= 2046
      VPAINT XS%(Stk% - 1), YS%(Stk% - 1), Colour%, Border%
      Stk% = Stk% - 1
    LOOP
    IF Y1% > 0 THEN
      NC% = VPOINT(I%, Y1% - 1)
      IF Border% = -1 THEN
        IF NC% = PColour% THEN
          IF M1% = -1 THEN M1% = I%
          NC% = VPOINT(I% + 1, Y1% - 1)
          IF I% = X2% OR NC% <> PColour% THEN
            XS%(Stk%) = (I% - M1%) \ 2 + M1%
            YS%(Stk%) = Y1% - 1
            Stk% = Stk% + 1
            M1% = -1
          END IF
        END IF
      ELSEIF NC% <> Border% AND NC% <> Colour% THEN
        IF M1% = -1 THEN M1% = I%
        NC% = VPOINT(I% + 1, Y1% - 1)
        IF I% = X2% OR NC% = Border% OR NC% = Colour% THEN
          XS%(Stk%) = (I% - M1%) \ 2 + M1%
          YS%(Stk%) = Y1% - 1
          Stk% = Stk% + 1
          M1% = -1
        END IF
      END IF
    END IF
    IF Y2% < ModeInfoBlock.HeightPixels - 1 THEN
      NC% = VPOINT(I%, Y2% + 1)
      IF Border% = -1 THEN
        IF NC% = PColour% THEN
          IF M2% = -1 THEN M2% = I%
          NC% = VPOINT(I% + 1, Y2% + 1)
          IF I% = X2% OR NC% <> PColour% THEN
            XS%(Stk%) = (I% - M2%) \ 2 + M2%
            YS%(Stk%) = Y2% + 1
            Stk% = Stk% + 1
            M2% = -1
          END IF
        END IF
      ELSEIF NC% <> Border% AND NC% <> Colour% THEN
        IF M2% = -1 THEN M2% = I%
        NC% = VPOINT(I% + 1, Y2% + 1)
        IF I% = X2% OR NC% = Border% OR NC% = Colour% THEN
          XS%(Stk%) = (I% - M2%) \ 2 + M2%
          YS%(Stk%) = Y2% + 1
          Stk% = Stk% + 1
          M2% = -1
        END IF
      END IF
    END IF
  NEXT I%
  M1% = -1: M2% = -1
  FOR I% = Y1% TO Y2%
    DO WHILE Stk% >= 2046
      VPAINT XS%(Stk% - 1), YS%(Stk% - 1), Colour%, Border%
      Stk% = Stk% - 1
    LOOP
    IF X1% > 0 THEN
      NC% = VPOINT(X1% - 1, I%)
      IF Border% = -1 THEN
        IF NC% = PColour% THEN
          IF M1% = -1 THEN M1% = I%
          NC% = VPOINT(X1% - 1, I% + 1)
          IF I% = Y2% OR NC% <> PColour% THEN
            XS%(Stk%) = X1% - 1
            YS%(Stk%) = (I% - M1%) \ 2 + M1%
            Stk% = Stk% + 1
            M1% = -1
          END IF
        END IF
      ELSEIF NC% <> Border% AND NC% <> Colour% THEN
        IF M1% = -1 THEN M1% = I%
        NC% = VPOINT(X1% - 1, I% + 1)
        IF I% = Y2% OR NC% = Border% OR NC% = Colour% THEN
          XS%(Stk%) = X1% - 1
          YS%(Stk%) = (I% - M1%) \ 2 + M1%
          Stk% = Stk% + 1
          M1% = -1
        END IF
      END IF
    END IF
    IF X2% < ModeInfoBlock.WidthPixels - 1 THEN
      NC% = VPOINT(X2% + 1, I%)
      IF Border% = -1 THEN
        IF NC% = PColour% THEN
          IF M2% = -1 THEN M2% = I%
          NC% = VPOINT(X2% + 1, I% + 1)
          IF I% = Y2% OR NC% <> PColour% THEN
            XS%(Stk%) = X2% + 1
            YS%(Stk%) = (I% - M2%) \ 2 + M2%
            Stk% = Stk% + 1
            M2% = -1
          END IF
        END IF
      ELSEIF NC% <> Border% AND NC% <> Colour% THEN
        IF M2% = -1 THEN M2% = I%
        NC% = VPOINT(X2% + 1, I% + 1)
        IF I% = Y2% OR NC% = Border% OR NC% = Colour% THEN
          XS%(Stk%) = X2% + 1
          YS%(Stk%) = (I% - M2%) \ 2 + M2%
          Stk% = Stk% + 1
          M2% = -1
        END IF
      END IF
    END IF
  NEXT I%
LOOP

END SUB

FUNCTION VPOINT% (X%, Y%)
SHARED ModeInfoBlock AS ModeInfoBlock
SHARED RefTable&(), CurrentBank%, WinSize&

Temp& = RefTable&(Y%) + X%
A% = Temp& \ WinSize&
IF A% <> CurrentBank% THEN
  SwitchBank A%
  CurrentBank% = A%
END IF
DEF SEG = &HA000
VPOINT% = PEEK(Temp& MOD WinSize&) AND 65535
DEF SEG

END FUNCTION

SUB VPRINT (Row%, Column%, Text$, Colour%)
SHARED Register AS RegTypeX

Register.AX = &H200
Register.DX = (Row% - 1) * 256 + (Column% - 1)
InterruptX2 &H10, Register, Register

Register.AX = &H1301
Register.BX = Colour%
Register.CX = LEN(Text$)
Register.ES = VARSEG(Text$)
Register.BP = SADD(Text$)
InterruptX2 &H10, Register, Register

END SUB

SUB VPRINT2 (Row%, Column%, Text$, Colour%, Lines%)
SHARED ModeInfoBlock AS ModeInfoBlock

'** REFERENCE:
'** (Row number in parentheses is for Lines% = 2)
'** Screen Size     Rows     Columns
'**  640 x 400     25 (50)      80
'**  640 x 480     30 (60)      80
'**  800 x 600     37 (75)     100
'** 1024 x 768     48 (96)     128

Column% = Column% - 1
Row% = Row% - 1
FOR I% = 1 TO LEN(Text$)
  IF Column% = ModeInfoBlock.WidthPixels \ 8 THEN
    Column% = 0
    Row% = Row% + 1
  END IF
  A$ = MID$(Text$, I%, 1)
  VTEXT Column% * 8, Row% * ((3 - Lines%) * 8), A$, Colour%, Lines%
  Column% = Column% + 1
NEXT I%

END SUB

SUB VPSET (X%, Y%, Colour%)
SHARED ModeInfoBlock AS ModeInfoBlock
SHARED RefTable&(), CurrentBank%, WinSize&

Temp& = RefTable&(Y%) + X%
A% = Temp& \ WinSize&
IF A% <> CurrentBank% THEN
  SwitchBank A%
  CurrentBank% = A%
END IF
DEF SEG = &HA000
POKE (Temp& MOD WinSize&) AND 65535, Colour%
DEF SEG

END SUB

SUB VTEXT (X%, Y%, Text$, Colour%, Lines%)
'** Modified from Douglas H. Lusher's "Gprint".

'** 8 x 8 char box, CGA character set.
CharSegment% = &HFFA6: CharOffset% = &HE

FOR Char% = 1 TO LEN(Text$)
  Ptr% = 8 * ASC(MID$(Text$, Char%, 1)) + CharOffset%
  FOR Ln% = 0 TO 7
    DEF SEG = CharSegment%
    BitPattern& = PEEK(Ptr% + Ln%) * 256&
    DEF SEG
    LineFormat% = (BitPattern& - 32768) XOR -32768
    VTLINE X%, Y% + Ln% * (3 - Lines%), Colour%, LineFormat%
    IF Lines% = 1 THEN VTLINE X%, Y% + Ln% * 2 + 1, Colour%, LineFormat%
  NEXT Ln%
  X% = X% + 8
NEXT Char%

END SUB

SUB VTLINE (X%, Y%, Colour%, LineFormat%)
'** Modified from Douglas H. Lusher's "Gprint".
SHARED ModeInfoBlock AS ModeInfoBlock
SHARED RefTable&(), CurrentBank%, WinSize&

Temp& = RefTable&(Y%) + X%
A% = Temp& \ WinSize&
IF A% <> CurrentBank% THEN
  SwitchBank A%
  CurrentBank% = A%
END IF
Temp2& = Temp& MOD WinSize&
TrueX% = Temp2& MOD 320
TrueY% = Temp2& \ 320
LINE (TrueX%, TrueY%)-(TrueX% + 7, TrueY%), Colour%, , LineFormat%

END SUB

