'** QDVTEST.BAS
'** QBasic source
'** May, 1997
'** DESCRIPTION: Accesses VESA VBE information, and writes it to a
'** file named QDVINFO.TXT, for diagnostic purposes.
'**
'** Stephen L. Maxson
'** smax@isc-durant.com
'**
'** CREDITS:
'** InterruptX2 procedure: Copyright (c) 1992, by Hans Lunsing.
'** VESA ModeInfoBlock structure "borrowed" from Nick Coons, who
'**   also provided other valuable information.
'**
'** Public Domain, (ab)use at your own risk
'**
'** Next three routines are required for VESA VBE calls.
DECLARE FUNCTION VesaStatus% ()
DECLARE SUB GetVESAInfo (Mode%)
DECLARE SUB InterruptX2 (IntNo%, Inreg AS ANY, OutReg AS ANY)

DEFINT A-Z

'** VESA VbeInfoBlock structure.
TYPE VbeInfoBlock
  VbeSignature          AS STRING * 4
  VbeVersion            AS INTEGER
  OemStringPtr          AS LONG
  Capabilities          AS STRING * 4
  VideoModePtr          AS LONG
  TotalMemory           AS INTEGER
  '** From here down are VESA VBE 2.0 additions.
  OemSoftwareRev        AS INTEGER
  OemVendorNamePtr      AS LONG
  OemProductNamePtr     AS LONG
  OemProductRevPtr      AS LONG
  Reserved              AS STRING * 222
  OemData               AS STRING * 256
END TYPE

'** VESA ModeInfoBlock structure.
TYPE ModeInfoBlock
  Attr                  AS INTEGER
  WinAAttr              AS STRING * 1
  WinBAttr              AS STRING * 1
  Gran                  AS INTEGER
  Size                  AS INTEGER
  WinASeg               AS INTEGER
  WinBSeg               AS INTEGER
  FarWindow             AS LONG
  BytesScanLine         AS INTEGER
  WidthPixels           AS INTEGER
  HeightPixels          AS INTEGER
  WidthChar             AS STRING * 1
  HeightChar            AS STRING * 1
  Planes                AS STRING * 1
  BitsPerPixel          AS STRING * 1
  Banks                 AS STRING * 1
  MemoryModel           AS STRING * 1
  BankSize              AS STRING * 1
  Pages                 AS STRING * 1
  Reserved              AS STRING * 1
  RedMask               AS STRING * 1
  RedField              AS STRING * 1
  GreenMask             AS STRING * 1
  GreenField            AS STRING * 1
  BlueMask              AS STRING * 1
  BlueField             AS STRING * 1
  ReservedMask          AS STRING * 1
  ReservedPosition      AS STRING * 1
  DirectColorInfo       AS STRING * 1
  VideoBuffer           AS LONG
  OffscreenMemory       AS LONG
  KBOffscreenMemory     AS INTEGER
  Reserved2             AS STRING * 206
END TYPE

'** Register type for use with InterruptX.
TYPE RegTypeX
  AX    AS INTEGER
  BX    AS INTEGER
  CX    AS INTEGER
  DX    AS INTEGER
  BP    AS INTEGER
  SI    AS INTEGER
  DI    AS INTEGER
  Flags AS INTEGER
  DS    AS INTEGER
  ES    AS INTEGER
END TYPE

CONST FALSE = 0, TRUE = NOT FALSE

DIM VbeInfoBlock AS VbeInfoBlock, ModeInfoBlock AS ModeInfoBlock
DIM Register AS RegTypeX
DIM VESAMode%(4)

VESAMode%(1) = &H100                     ' 640x400x256
VESAMode%(2) = &H101                     ' 640x480x256
VESAMode%(3) = &H103                     ' 800x600x256
VESAMode%(4) = &H105                     '1024x768x256

SCREEN 0, 0, 0
WIDTH 80, 25
CLS

IF VesaStatus% = 0 THEN
  PRINT "VESA VGA Bios Extensions do not appear to be available."
  PRINT "Install the appropriate VESA VBE driver for your video card,"
  PRINT "and try again."
  END
END IF
Version$ = RTRIM$(STR$(VbeInfoBlock.VbeVersion \ 256)) + "."
Version$ = Version$ + LTRIM$(STR$(VbeInfoBlock.VbeVersion MOD 256))

VM% = 2
'** Get the info on the selected mode.
GetVESAInfo VM%
WinSize& = ModeInfoBlock.Size * 1024&
Temp& = ModeInfoBlock.HeightPixels
Temp& = Temp& * ModeInfoBlock.BytesScanLine
IF Temp& MOD WinSize& <> 0 THEN NumWindows% = 1
NumWindows% = NumWindows% + Temp& \ WinSize&


OPEN "QDVINFO.TXT" FOR OUTPUT AS #1
PRINT #1, "Information from VbeInfoBlock:"
PRINT #1, ""
PRINT #1, "VbeSignature:      "; VbeInfoBlock.VbeSignature
PRINT #1, "VbeVersion:        "; Version$
PRINT #1, "Capabilities:      "; CVL(VbeInfoBlock.Capabilities)
PRINT #1, "VideoModePtr:      "; VbeInfoBlock.VideoModePtr
PRINT #1, "TotalMemory:       "; VbeInfoBlock.TotalMemory
PRINT #1, ""
PRINT #1, ""
PRINT #1, "Information from ModeInfoBlock:"
PRINT #1, ""
PRINT #1, "Attr:              "; ModeInfoBlock.Attr
PRINT #1, "WinAAttr:          "; ASC(ModeInfoBlock.WinAAttr)
PRINT #1, "WinBAttr:          "; ASC(ModeInfoBlock.WinBAttr)
PRINT #1, "Gran:              "; ModeInfoBlock.Gran
PRINT #1, "Size:              "; ModeInfoBlock.Size
PRINT #1, "WinASeg:           "; ModeInfoBlock.WinASeg
PRINT #1, "WinBSeg:           "; ModeInfoBlock.WinBSeg
PRINT #1, "FarWindow:         "; ModeInfoBlock.FarWindow
PRINT #1, "BytesScanLine:     "; ModeInfoBlock.BytesScanLine
PRINT #1, "WidthPixels:       "; ModeInfoBlock.WidthPixels
PRINT #1, "HeightPixels:      "; ModeInfoBlock.HeightPixels
PRINT #1, "Banks:             "; ASC(ModeInfoBlock.Banks)
PRINT #1, "MemoryModel:       "; ASC(ModeInfoBlock.MemoryModel)
PRINT #1, "BankSize:          "; ASC(ModeInfoBlock.BankSize)
PRINT #1, "Pages:             "; ASC(ModeInfoBlock.Pages)
PRINT #1, "VideoBuffer:       "; ModeInfoBlock.VideoBuffer
PRINT #1, "OffscreenMemory:   "; ModeInfoBlock.OffscreenMemory
PRINT #1, "KBOffscreenMemory: "; ModeInfoBlock.KBOffscreenMemory
PRINT #1, ""
PRINT #1, ""
PRINT #1, "Determined information:"
PRINT #1, ""
PRINT #1, "WinSize:           "; WinSize&
PRINT #1, "NumWindows:        "; NumWindows%
CLOSE #1
PRINT : PRINT "QDVINFO.TXT created."

END

'-----------------------------------------------------------------------
' The machine code of the interrupt procedure for InterruptX2
'-----------------------------------------------------------------------
InterruptXASM:
' Number of bytes
DATA  190
' Hexadecimal representation of machine code
DATA 55,8B,EC,8B,5E,0C,8B,17,0A,F6
DATA 74,07,C7,07,FF,FF,E9,A7,00,8B
DATA 5E,06,8B,1F,2E,88,97,77,00,32
DATA C0,80,FA,25,74,05,80,FA,26,75
DATA 02,0C,02,50,1E,06,56,57,9C,8B
DATA 76,0A,80,FA,20,7C,05,80,FA,30
DATA 7C,0A,81,7C,08,FF,FF,74,03,8B
DATA 6C,08,8B,44,0E,25,D5,0F,50,8B
DATA 04,8B,5C,02,8B,4C,04,8B,54,06
DATA 8B,7C,0C,FF,74,0A,81,7C,12,FF
DATA FF,74,03,8E,44,12,81,7C,10,FF
DATA FF,74,03,8E,5C,10,5E,9D,CD,00
DATA 55,8B,EC,9C,83,C5,0E,F6,46,FE
DATA 02,74,02,45,45,1E,56,8E,5E,FC
DATA 8B,76,08,89,04,89,5C,02,89,4C
DATA 04,89,54,06,8F,44,0A,89,7C,0C
DATA 8F,44,10,8C,44,12,8F,44,0E,8F
DATA 44,08,F6,46,FE,02,74,02,44,44
DATA 9D,5F,5E,07,1F,58,5D,CA,08,00

SUB GetVESAInfo (Mode%)
SHARED VESAMode%(), Register AS RegTypeX, ModeInfoBlock AS ModeInfoBlock

Register.AX = &H4F01                    '4F01H - Get VESA Mode Information
Register.CX = VESAMode%(Mode%)           'Selected Mode
Register.ES = VARSEG(ModeInfoBlock)     'Where InfoBlock is to be stored
Register.DI = VARPTR(ModeInfoBlock)     '  "       "      "  "  "   "
InterruptX2 &H10, Register, Register

END SUB

SUB InterruptX2 (IntNo%, Inreg AS RegTypeX, OutReg AS RegTypeX) STATIC

'-----------------------------------------------------------------------
' InterruptX2
' Interrupt procedure for QBASIC
' (c) 1992 Hans Lunsing
' ----------------------------------------------------------------------

'-----------------------------------------------------------------------
' Interrupt procedure. Works in the same way as its QB 4.5 counterpart.
'-----------------------------------------------------------------------

IF NOT MachineCode% THEN
  ' First time dimension string array to hold machine code
  RESTORE InterruptXASM
  READ nASMBYTES%
  REDIM ASMBuffer(0 TO nASMBYTES% - 1) AS STRING * 1
END IF

' Get address of machine code
DEF SEG = VARSEG(ASMBuffer(0))
offset% = VARPTR(ASMBuffer(0))

IF NOT MachineCode% THEN
  ' First time load string array with machine code
  FOR I% = 0 TO nASMBYTES% - 1
    READ code$
    POKE offset% + I%, VAL("&H" + code$)
  NEXT I%
  ' Indicate availability of machine code
  MachineCode% = TRUE
END IF

' Call interrupt.
' The first Offset% parameter is used by the machine code modifying itself,
' so don't leave it out.
CALL Absolute(IntNo%, Inreg, OutReg, offset%, offset%)

DEF SEG

END SUB

FUNCTION VesaStatus%
SHARED Register AS RegTypeX, VbeInfoBlock AS VbeInfoBlock

VbeInfoBlock.VbeSignature = "VBE2"      'Return 2.0 info, if available
Register.AX = &H4F00                    '4F00H - Get VESA VBE Information
Register.ES = VARSEG(VbeInfoBlock)      'Where InfoBlock is to be stored
Register.DI = VARPTR(VbeInfoBlock)      '  "       "      "  "  "   "
InterruptX2 &H10, Register, Register
IF Register.AX = &H4F AND VbeInfoBlock.VbeSignature = "VESA" THEN
  VesaStatus% = -1    'QBasic TRUE
ELSE
  VesaStatus% = 0     'QBasic FALSE
END IF

END FUNCTION

