'**************************************************************************
' Qmouse.bas  a program to demonstrate mouse support in qbasic programs.  *
' Written by: Kerry S. Goodin, D.D.S. (updated 8-98)                      *
' uses an assembler program to call Interrupts to query the mouse driver. *
'                                                                         *
' Shows the use of an assembler program that mimics                       *
' the QuickBasic interrupt SUB in QB quick library.                       *
' Now Qbasic programs can call any interrupt, not just a mouse            *
' by changing the IntNumber in the SUB InterruptQ.                        *
'                                                                         *
' to use in QB4.5 load with the /lQB switch (Call Absolute                *
' is needed for QuickBasic. It is already loaded in qbasic.)              *
'**************************************************************************


DECLARE SUB InterruptQ (IntNumber AS INTEGER, inRegs AS ANY, outRegs AS ANY)
DECLARE SUB ShowMouse ()
DECLARE SUB HideMouse ()
DECLARE SUB MOUSE (a%, b%, c%, d%)
DECLARE SUB LimitMouse (ULX%, ULY%, LRX%, LRY%)
DECLARE SUB MouseStatusNow (LeftButton%, RightButton%, xMouse%, yMouse%)
DECLARE SUB GetLeftPress (LeftCount%, xMouse%, yMouse%)
DECLARE SUB GetRightpress (RightCount%, xMouse%, yMouse%)
DECLARE SUB GetLeftRelease (LeftCount%, xMouse%, yMouse%)
DECLARE SUB GetRightRelease (RightCount%, xMouse%, yMouse%)
DECLARE SUB InitalizeMouse (MouseExist%)

DEFINT A-Z

CONST False = 0, True = NOT False


' Define INTERRUPT REGISTERS

TYPE RegType
    ax    AS INTEGER
    bx    AS INTEGER
    cx    AS INTEGER
    dx    AS INTEGER
    bp    AS INTEGER
    si    AS INTEGER
    di    AS INTEGER
    Flags AS INTEGER
    ds    AS INTEGER
    es    AS INTEGER
END TYPE

DIM SHARED inRegs AS RegType
DIM SHARED outRegs AS RegType
REDIM SHARED Assembler(1 TO 50)


' Places Assembler Program into Memory

DEF SEG = VARSEG(Assembler(1))
Address = VARPTR(Assembler(1))
FOR I = 0 TO 99
        READ a
        POKE Address + I, a
NEXT I



'check to see if mouse present,if so returns True
'or if not then returns False

CALL InitalizeMouse(MouseExist)

'start of sample program code

CLS

IF MouseExist THEN
    PRINT "mouse present"
    CALL ShowMouse                      ' Makes Mouse Appear on Screen
    
    PRINT "press left button"           ' Test of Press Left Button
    DO
        CALL GetLeftPress(LeftCount, xMouse, yMouse)
    LOOP WHILE LeftCount = 0
    
    PRINT "press right button"          ' Test of Press Right Button
    DO
        CALL GetRightpress(RightCount, xMouse, yMouse)
    LOOP WHILE RightCount = 0
    
    CALL InitalizeMouse(Exist)          ' Reinitialize Mouse
    CALL ShowMouse                      ' Makes Mouse appear
    
    PRINT "release left button"         ' Test the Release of Left Button
    DO
        CALL GetLeftRelease(LeftCount, xMouse, yMouse)
    LOOP WHILE LeftCount = 0
    
    PRINT "release right button"        ' Test the Release of Right Button
    DO
        CALL GetRightRelease(RightCount, xMouse, yMouse)
    LOOP WHILE RightCount = 0
    
    PRINT "Mouse now has limited range"
    PRINT "hit ESC to end"              ' Test of the Button Status and Position of Mouse
    CALL LimitMouse(100, 100, 200, 200) ' Limits the Mouse to a Box
    DO
        X$ = INKEY$
        CALL MouseStatusNow(LeftButtonPress, RightButtonPress, xMouse, yMouse)
        LOCATE 10, 10
        PRINT "left:"; LeftButtonPress; " right:"; RightButtonPress;
        LOCATE 11, 10
        PRINT "   X:"; xMouse; "   Y:"; yMouse;
    LOOP UNTIL X$ = CHR$(27)
    CALL HideMouse                      ' Makes Mouse disappear
    
ELSE
    PRINT " no mouse present"           ' No Mouse was found
END IF

END


'Data for the Assembler Program

DATA 85,139,236,86,87,30,139,118,6,139,4,139,92,2,139,76,4,139,84,6,139
DATA 108,8,139,124,12,142,68,18,255,116,10,131,124,18,255,117,2,30,7
DATA 131,124,16,255,116,3,142,92,16,94,205,33,85,139,236,30,86,142,94,2
DATA 139,118,14,137,4,137,92,2,137,76,4,137,84,6,143,68,10,143,68,16
DATA 143,68,8,137,124,12,140,68,18,156,143,68,14,95,95,94,93,202,2,0

' ******************************************
' * Returns the # of times the left button *
' * has been pressed since the last call   *
' * and the Position of the mouse in x & y *
' ******************************************
'
'
'
SUB GetLeftPress (LeftCount, xMouse, yMouse)
  a = 5
  LeftCount = 0
  
  CALL MOUSE(a, LeftCount, xMouse, yMouse)
  
END SUB

' ******************************************
' * Returns the # of times the left button *
' * has been released since the last call  *
' * and the Position of the mouse in x & y *
' ******************************************
'
'
SUB GetLeftRelease (LeftCount, xMouse, yMouse)
  a = 6
  LeftCount = 0
  
  CALL MOUSE(a, LeftCount, xMouse, yMouse)
  
END SUB

' ******************************************
' * Returns the # of times the right button*
' * has been pressed since the last call   *
' * and the Position of the mouse in x & y *
' ******************************************
'
'
SUB GetRightpress (RightCount, xMouse, yMouse)
  a = 5
  RightCount = 1
  
  CALL MOUSE(a, RightCount, xMouse, yMouse)
  
END SUB

' ******************************************
' * Returns the # of times the right button*
' * has been released since the last call  *
' * and the Position of the mouse in x & y *
' ******************************************
'
'
SUB GetRightRelease (RightCount, xMouse, yMouse)
  a = 6
  RightCount = 1
  
  CALL MOUSE(a, RightCount, xMouse, yMouse)
  
END SUB

' ****************************************
' * hides the mouse cursor on the screen *
' ****************************************
'
'
SUB HideMouse
  a = 2
  
  CALL MOUSE(a, 0, 0, 0)
  
END SUB

' *************************************
' * intializes and checks for a mouse *
' * returns True if present and False *
' * if no mouse drive if found        *
' *************************************
'
'
SUB InitalizeMouse (Exist)
  a = 0
  
  CALL MOUSE(a, 0, 0, 0)
  
  IF a THEN
      Exist = True                        ' mouse installed
  ELSE
      Exist = False                       ' no mouse installed
  END IF
  
END SUB

' ************************************
' * Calls an interrupt from the call *
' * absolute array read into memory  *
' ************************************
'
'
SUB InterruptQ (IntNumber AS INTEGER, inRegs AS RegType, outRegs AS RegType)

  'keep the value of inReg
  outRegs = inRegs

  DEF SEG = VARSEG(Assembler(1))
  Address = VARPTR(Assembler(1))
  POKE Address + 51, IntNumber

  CALL ABSOLUTE(outRegs, Address)

END SUB

'***************************************
'* Sub to limit the range of the mouse *
'***************************************
'
' ULX-upper left corner x coordinates
' ULY-upper left corner y coordinates
' LRX-lower right corner x coordinates
' LRY-lower right corner y coordinates
'
SUB LimitMouse (ULX, ULY, LRX, LRY)
  
  CALL MOUSE(7, 0, ULX, LRX)
  CALL MOUSE(8, 0, ULY, LRY)
  
  
END SUB

'**********************************************
'* Sub Program to place the correct values    *
'* into CPU registers for the call InterruptQ *
'* then replaces the returned values from CPU *
'* Registers into the values needed for the   *
'* subprogram that called mouse               *
'**********************************************
'
'
SUB MOUSE (a, b, c, d)
  
  'place values into CPU registers
  
  inRegs.ax = a
  inRegs.bx = b
  inRegs.cx = c
  inRegs.dx = d
  
  'set flag for interrupt 33H  for the mouse
  IntNumber = &H33
  
  CALL InterruptQ(IntNumber, inRegs, outRegs)
  
  'get values from CPU register
  
  a = outRegs.ax
  b = outRegs.bx
  c = outRegs.cx
  d = outRegs.dx
  
END SUB

' ******************************************
' * Returns True if the left button is     *
' * pressed.  Returns True if Right        *
' * Button is Pressed, and the Position of *
' * the mouse x & y.                       *
' ******************************************
'
'
SUB MouseStatusNow (LeftButtonPress, RightButtonPress, xMouse, yMouse)
  a = 3
  b = 0
  
  CALL MOUSE(a, b, xMouse, yMouse)
  
  LeftButtonPress = ((b AND 1) <> 0)
  RightButtonPress = ((b AND 2) <> 0)
  
END SUB

' *********************
' * shows the Mouse   *
' *********************
'
'
SUB ShowMouse
  a = 1
  
  CALL MOUSE(a, 0, 0, 0)
  
END SUB

