'** ROTDS.BAS
'** QBasic source
'** June, 1997
'**
'** DESCRIPTION:
'** Manipulates a sprite composed of directional DRAW commands,
'** stored in a string variable.
'** The left and right arrow keys rotate the sprite.
'** The up and down arrow keys move the sprite forward and back.
'** The A and Z keys increase/decrease rotation speed.
'** The <Esc> key exits.
'**
'** Stephen L. Maxson
'** smax@isc-durant.com
'**
'** CREDITS:
'** Thanks to everyone in alt.lang.basic and comp.lang.basic.misc
'** who helped me out with finding radial directions using the SIN
'** and COS functions.
'**
DECLARE FUNCTION CheckWrapX% (X%)
DECLARE FUNCTION CheckWrapY% (Y%)
DECLARE SUB DrawSprite (Graphic$, X%, Y%, Rotation%)
DEFINT A-Z

PSpeed% = 0
PRotSpeed% = 10
PX% = 160
PY% = 100
'** Array for background saving.
DIM BGsave%(614)
'** Create tables for movement.
DIM SINtable!(359), COStable!(359)
RadPerDeg! = ATN(1) * 4 / 180
FOR I% = 0 TO 359
  SINtable!(I%) = SIN(I% * RadPerDeg!)
  COStable!(I%) = COS(I% * RadPerDeg!)
NEXT I%

'** EGA, 320 x 200 x 16.
'** Draw on page 1, view page 0.
SCREEN 7, , 1, 0
'** We need some stars for a background.
FOR I% = 0 TO 200
  X% = INT(RND * 320)
  Y% = INT(RND * 200)
  SELECT CASE INT(RND * 3)
    CASE 0
      PSET (X%, Y%), 8
    CASE 1
      PSET (X%, Y%), 7
    CASE 2
      PSET (X%, Y%), 15
  END SELECT
NEXT I%
'** Get the sprites (only one in this program).
RESTORE SpriteData
READ N%
DIM Sprite$(N%)
FOR I% = 0 TO N%
  READ K%
  FOR J% = 0 TO K%
    READ A$
    Sprite$(I%) = Sprite$(I%) + A$
  NEXT J%
NEXT I%
'** GET background and draw sprite for first time.
GET (PX% - 17, PY% - 17)-(PX% + 17, PY% + 17), BGsave%
DrawSprite Sprite$(0), PX%, PY%, PRot%
'** Move to visible page first time.
PCOPY 1, 0

'** Main loop starts here.
DO
  '** PUT background back to erase sprite.
  PUT (PX% - 17, PY% - 17), BGsave%, PSET
  A$ = UCASE$(INKEY$)
  '** Key pressed?
  IF LEN(A$) THEN
    A% = ASC(RIGHT$(A$, 1))
    '** Clear keyboard buffer.
    DEF SEG = &H40         'switch to BIOS data segment
    POKE &H1A, PEEK(&H1C)  '\ make KBbuffer head pointer
    POKE &H1B, PEEK(&H1D)  '/ equal KBbuffer tail pointer
    DEF SEG                'back to default data segment
    '** Extended key pressed?
    IF LEN(A$) = 2 THEN
      SELECT CASE A%
        CASE 72 'Up (forward)
          PSpeed% = PSpeed% + 1
          IF PSpeed% > 20 THEN PSpeed% = 20
        CASE 75 'Left
          PRot% = PRot% + PRotSpeed%
          IF PRot% > 359 THEN PRot% = PRot% - 360
        CASE 77 'Right
          PRot% = PRot% - PRotSpeed%
          IF PRot% < 0 THEN PRot% = PRot% + 360
        CASE 80 'Down (reverse)
          PSpeed% = PSpeed% - 1
          IF PSpeed% < -20 THEN PSpeed% = -20
      END SELECT
    '** Other (non-extended) keys.
    ELSE
      SELECT CASE A%
        CASE 65 'A
          PRotSpeed% = PRotSpeed% + 1
          IF PRotSpeed% > 45 THEN PRotSpeed% = 45
        CASE 90 'Z
          PRotSpeed% = PRotSpeed% - 1
          IF PRotSpeed% < 1 THEN PRotSpeed% = 1
      END SELECT
    END IF
  END IF
  IF PSpeed% <> 0 THEN
    PX% = CheckWrapX%(PX% - SINtable!(PRot%) * PSpeed%)
    PY% = CheckWrapY%(PY% - COStable!(PRot%) * PSpeed%)
  END IF
  '** GET new background.
  GET (PX% - 17, PY% - 17)-(PX% + 17, PY% + 17), BGsave%
  '** DRAW sprite in new position.
  DrawSprite Sprite$(0), PX%, PY%, PRot%
  '** Move to visible page.
  PCOPY 1, 0
'** Exit loop if <Esc> pressed.
LOOP UNTIL A$ = CHR$(27)
END

SpriteData:
'** Number of sprites.
DATA 0
'** Number of lines for this sprite.
DATA 12
DATA "BH7"
DATA "BD10C3E3UE2UE2U"
DATA "DF2DF2DF3"
DATA "D4L14U4R14"
DATA "BH7P11,3"
DATA "BD9P11,3"
DATA "BU3BH3C7E2R2F2D2G2L2H2U2"
DATA "BU2BF3P15,7"
DATA "BHC7R2D2L2U2BFP7,7"
DATA "BG6BD3C8L2HU10ER2FD10G"
DATA "BHBU3P7,8"
DATA "BF4BR9C8R2EU10HL2GD10F"
DATA "BEBU3P7,8"
'** 15, 12 (last position of drawing cursor, for my own use).

FUNCTION CheckWrapX% (X%)

IF X% < 20 THEN X% = 280 + X%
IF X% > 299 THEN X% = X% - 280
CheckWrapX% = X%

END FUNCTION

FUNCTION CheckWrapY% (Y%)

IF Y% < 20 THEN Y% = 160 + Y%
IF Y% > 179 THEN Y% = Y% - 160
CheckWrapY% = Y%

END FUNCTION

SUB DrawSprite (Graphic$, X%, Y%, Rotation%)

'** Move to current position on screen.
G$ = "BM" + LTRIM$(STR$(X%)) + "," + LTRIM$(STR$(Y%))
'** Rotational command.
G$ = "TA" + LTRIM$(STR$(Rotation%)) + G$ + Graphic$
'** Draw complete sprite string.
DRAW "X" + VARPTR$(G$)

END SUB

