'** SCRTILE.BAS
'** QBasic source
'** October, 1997
'** Description:
'** Single-pixel tile scroller, like the type used in overhead RPGs.
'** Uses 24 x 24 graphic tiles, on a 16 x 16 map.  A 13 x 7 array of
'** tiles is displayed on the screen.  User scrolls around the map by
'** using the arrow keys. The 'R' key toggles between slow and fast
'** walking speed.  The <Esc> key exits.
'**
'** Stephen L. Maxson
'** smax@isc-durant.com
'**
'** CREDITS:
'** The EMS routines and the fast MemCopy routine are by Jonathan
'**   Leger.
'**
'** Thanks to Andrew L. Ayers for info on sprite width being
'**   stored multiplied by 8.
'**
'** Public Domain, (ab)use at your own risk
'**
'** Leger's EMS routines.
DECLARE FUNCTION NumEMSHandles% ()
DECLARE FUNCTION GetEMS% (numpages%)
DECLARE FUNCTION EMSPages% (func%)
DECLARE FUNCTION PageFrame% ()
DECLARE FUNCTION EMSStatus% ()
DECLARE SUB ReleaseEMS (Handle%)
DECLARE SUB MapEMS (Handle%, pageoffset%)
'** Leger's fast memory copying routine.
DECLARE SUB MemCopy (fromseg%, fromoff%, toseg%, tooff%, bytes%)
'** My stuff.
DECLARE FUNCTION fGetKey% ()
DECLARE FUNCTION CheckDirection% (Direction%)
DECLARE SUB MoveScreen (Direction%, ScrollStep%)
DECLARE SUB DrawScreen ()
DECLARE SUB KillPalette ()
DECLARE SUB RotatePalette (BeginColour%, EndColour%)
DECLARE SUB RotTimer ()
DECLARE SUB ClipArray (HostArray%(), NewArray%(), X1%, Y1%, X2%, Y2%)
DECLARE SUB InsertArray (SourceArray%(), TargetArray%(), X%, Y%)
DECLARE SUB SpriteToScreen (Array%(), X%, Y%)
DECLARE SUB EMSGET (X1%, Y1%, X2%, Y2%)
DECLARE SUB EMSPUT (X%, Y%, ImageNum%, Mode$)
DECLARE SUB EMSPULL (Array%(), ImageNum%)
DEFINT A-Z

MaxImages% = 4
MaxPages% = 2
'** User-defined data type to store where the images are in EMS.
TYPE EMSImageIndex
  ImagePage AS INTEGER
  ImageAddress AS INTEGER
  ImageSize AS INTEGER
END TYPE
DIM ImageIndex(MaxImages%) AS EMSImageIndex

SCREEN 0
WIDTH 80, 25
CLS
'** EMS available?
IF NOT EMSStatus% THEN
  BEEP
  PRINT "EMS does not appear to be available."
  END
END IF
'** Is an unused handle available?
IF NumEMSHandles% > 255 THEN
  BEEP
  PRINT "No free EMS handles available."
  END
END IF
'** Enough EMS available?
IF EMSPages%(1) < MaxPages% THEN
  BEEP
  PRINT "Not enough EMS memory available."
  PRINT "Requested: "; MaxPages% * 16384&
  PRINT "Available: "; EMSPages%(1) * 16384&
  END
END IF
'** Get 2 16K pages of EMS.
EMSHandle% = GetEMS%(MaxPages%)
'** Get the page frame, the address we'll write to in memory.
EMSSegment% = PageFrame%

'** Number of pixels scrolled per step. Must be even divisor of
'** tile size (24).
PlayerSpeed% = 3

'** Graphic arrays for our character, a mask for transparency,
'** and a buffer for saving the background.  Stored in main
'** memory rather than EMS only becuase it's easier for me to
'** remember.
DIM Character%(290), SaveBuffer%(290)
'** Our map is 16 x 16.
DIM Map%(15, 15)
'** Read in the map.
RESTORE MapData
FOR I% = 0 TO 15
  FOR J% = 0 TO 15
    READ Map%(J%, I%)
  NEXT J%
NEXT I%
'320x200x256 graphics.
SCREEN 13
'** Set entire palette to black, so our drawing is invisible.
KillPalette
'** Graphics time!
'** Draw Character (Stickman the Barbarian).
CIRCLE (12, 5), 3, 8, , , 1
LINE (12, 9)-(12, 18), 8
LINE (11, 19)-(8, 22), 8
LINE (13, 19)-(16, 22), 8
CIRCLE (16, 13), 3, 8, , , 3
LINE (13, 10)-(14, 10), 8
LINE (11, 10)-(7, 13), 8
LINE (7, 5)-(7, 14), 8
LINE (6, 13)-(8, 13), 8
GET (0, 0)-(23, 23), Character%
'** The following are our tiles.
'** Water.
J% = 0
FOR I% = 0 TO 23
  LINE (0, I%)-(I%, 0), J% + 16
  LINE (I% + 1, 23)-(23, I% + 1), J% + 16
  J% = (J% + 1) MOD 6
NEXT I%
EMSGET 0, 0, 23, 23
'** Grass.
LINE (0, 0)-(23, 23), 10, BF
FOR I% = 0 TO 5
  FOR J% = 0 TO 5
    PSET (I% * 4, J% * 4), 2
    PSET (I% * 4 + 2, J% * 4 + 2), 2
  NEXT J%
NEXT I%
EMSGET 0, 0, 23, 23
'** Desert.
LINE (0, 0)-(23, 23), 14, BF
FOR I% = 0 TO 23
  FOR J% = 0 TO 5
    PSET (J% * 4 + (I% MOD 2), I%), 7
    PSET ((J% + 1) * 4 - (I% MOD 4) - 1, I%), 7
  NEXT J%
NEXT I%
EMSGET 0, 0, 23, 23
'** Forest.
LINE (0, 0)-(23, 23), 2, BF
FOR I% = 0 TO 1
  I2% = I% MOD 2
  FOR J% = 0 TO 1
    J2% = J% MOD 2
    CIRCLE (J% * 12 + I2% + 5, I% * 12 + J2% + 5), 5, 10, , , 1
    PAINT (J% * 12 + I2% + 5, I% * 12 + J2% + 5), 10, 10
  NEXT J%
NEXT I%
FOR I% = 0 TO 23
  FOR J% = 0 TO 11
    PSET (J% * 2 + (I% MOD 2), I%), 2
  NEXT J%
NEXT I%
EMSGET 0, 0, 23, 23
'** Mountain.
LINE (0, 0)-(23, 23), 6, BF
LINE (0, 0)-(11, 11), 8
LINE (1, 23)-(19, 5), 8
LINE (20, 6)-(23, 9), 8
LINE (0, 10)-(6, 16), 8
LINE (8, 8)-(16, 0), 8
LINE (17, 23)-(20, 20), 8
LINE (21, 21)-(23, 23), 8
EMSGET 0, 0, 23, 23
'** Clear the screen, perform initial setup.
CLS
'** Viewport border.
LINE (0, 0)-(319, 175), 3, B
LINE (1, 1)-(318, 174), 11, B
LINE (2, 2)-(317, 173), 11, B
LINE (3, 3)-(316, 172), 3, B
PlayerX% = 7
PlayerY% = 7
'** Program just started, so we need to draw the entire screen.
DrawScreen
'** Restore palette colors.
PALETTE
'** Set up rotating palette for water.
FOR I% = 0 TO 7
  OUT &H3C8, 16 + I%
  OUT &H3C9, 0
  OUT &H3C9, I% * 8
  OUT &H3C9, 63
NEXT I%
'** Save the background, then draw our character for the first time.
GET (148, 76)-(171, 99), SaveBuffer%
SpriteToScreen Character%(), 148, 76
'** Below is the main control loop.
DO
  '** Get a keypress.
  A% = fGetKey%
  '** Was the key an extended key?
  IF A% < 0 THEN
    '** Extended key, let's get the value.
    '** Was it a directional key?
    IF A% = -72 OR A% = -75 OR A% = -77 OR A% = -80 THEN
      '** Directional key, find appropriate direction.
      SELECT CASE A%
        CASE -72
          Direction% = 0  '** North
        CASE -75
          Direction% = 3  '** West
        CASE -77
          Direction% = 2  '** East
        CASE -80
          Direction% = 1  '** South
      END SELECT
      '** What does the map show in that direction?
      J% = CheckDirection%(Direction%)
      '** Unless it's water (0) or mountain (4), we can go there.
      IF J% <> 0 AND J% <> 4 THEN
        '** Scroll 24 pixels, the length of a single tile.
        FOR I% = 0 TO 24 - PlayerSpeed% STEP PlayerSpeed%
          '** We pass the direction and where we are in the
          '** scroll to our scrolling SUB.
          MoveScreen Direction%, I%
          '** Next 4 lines clear the keyboard buffer, so we can hold
          '** down a key without that annoying chirping noise.
          DEF SEG = &H40         'switch to BIOS data segment
          POKE &H1A, PEEK(&H1C)  '\ make KBbuffer head pointer
          POKE &H1B, PEEK(&H1D)  '/ equal KBbuffer tail pointer
          DEF SEG                'back to default data segment
        NEXT I%
        '** Scroll has occurred, so let's update our position on
        '** the map.  Correct the value if it wraps-around the map.
        SELECT CASE Direction%
          CASE 0  '** North
            PlayerY% = PlayerY% - 1
            IF PlayerY% = -1 THEN PlayerY% = 15
          CASE 1  '** South
            PlayerY% = PlayerY% + 1
            IF PlayerY% = 16 THEN PlayerY% = 0
          CASE 2  '** East
            PlayerX% = PlayerX% + 1
            IF PlayerX% = 16 THEN PlayerX% = 0
          CASE 3  '** West
            PlayerX% = PlayerX% - 1
            IF PlayerX% = -1 THEN PlayerX% = 15
        END SELECT
      '** Attempting to move into water or mountain?
      ELSE
        '** An annoying beep (better than drowning, or knocking
        '** yourself silly).
        SOUND 500, .5
      END IF
    END IF
  END IF
  '** Did they press "R"?  If so, toggle speed.
  IF A% = 82 OR A% = 114 THEN PlayerSpeed% = 9 - PlayerSpeed%
'** Exit main control loop if <Esc> is pressed.
LOOP UNTIL A% = 27
'** Release the EMS memory.
ReleaseEMS EMSHandle%
'** I refuse to tell you what this next statement does.
END

'** Here's the data for our 16 x 16 map.
'** 0 = Water     1 = Grass     2 = Desert
'** 3 = Forest    4 = Mountain
MapData:
DATA 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3
DATA 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 3, 3, 3, 1
DATA 0, 0, 0, 0, 0, 0, 1, 1, 1, 3, 1, 1, 1, 1, 1, 1
DATA 0, 0, 0, 0, 0, 1, 1, 1, 3, 3, 3, 1, 1, 1, 0, 0
DATA 0, 0, 0, 1, 1, 1, 1, 1, 3, 3, 1, 1, 0, 0, 0, 0
DATA 0, 0, 1, 1, 3, 3, 3, 1, 1, 1, 1, 1, 0, 0, 0, 0
DATA 0, 1, 1, 3, 3, 3, 3, 1, 4, 4, 1, 1, 0, 0, 0, 0
DATA 0, 1, 1, 3, 0, 0, 3, 3, 4, 4, 2, 1, 0, 0, 0, 0
DATA 0, 0, 1, 1, 0, 0, 0, 4, 4, 2, 2, 2, 1, 0, 0, 0
DATA 0, 0, 0, 1, 1, 0, 0, 1, 2, 2, 2, 2, 1, 0, 0, 0
DATA 0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 0, 0, 0
DATA 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0
DATA 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0
DATA 0, 1, 1, 4, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
DATA 1, 3, 3, 4, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
DATA 3, 3, 3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1

FUNCTION CheckDirection% (Direction%)
SHARED Map%(), PlayerX%, PlayerY%

'** CheckDirection% returns the value of what's in the map square
'** in the direction contained in Direction% from the player's
'** current position.

I% = PlayerX%
J% = PlayerY%
SELECT CASE Direction%
  CASE 0  '** North
    J% = PlayerY% - 1
  CASE 1  '** South
    J% = PlayerY% + 1
  CASE 2  '** East
    I% = PlayerX% + 1
  CASE 3  '** West
    I% = PlayerX% - 1
END SELECT
'** Correct values if they wrap-around the map.
IF I% = -1 THEN I% = 15
IF J% = -1 THEN J% = 15
IF I% = 16 THEN I% = 0
IF J% = 16 THEN J% = 0
CheckDirection% = Map%(I%, J%)

END FUNCTION

SUB ClipArray (HostArray%(), NewArray%(), X1%, Y1%, X2%, Y2%)

'** First byte of array (0) holds width information,
'** multiplied by 8.
HostWidth% = HostArray%(0) \ 8
'** Find width of new array.
NewWidth% = X2% - X1% + 1
'** Put it in the new array.
NewArray%(0) = NewWidth% * 8
'** Put height of new array in.
NewArray%(1) = Y2% - Y1% + 1
'** Find data segments and offsets for the two arrays.
HostSeg% = VARSEG(HostArray%(0))
HostOff% = VARPTR(HostArray%(0))
NewSeg% = VARSEG(NewArray%(0))
NewOff% = VARPTR(NewArray%(0))
'** Outer loop steps through height.
FOR I% = Y1% TO Y2%
  '** Next two statements reduce math operations performed
  '** inside inner loop.
  HostI% = I% * HostWidth%
  NewI% = (I% - Y1%) * NewWidth%
  '** Inner loop steps through width.
  FOR J% = X1% TO X2%
    '** Switch to host array's data segment.
    DEF SEG = HostSeg%
    '** Pull a single byte/pixel from host array.
    Byte% = PEEK(HostOff% + 4 + HostI% + J%)
    '** Switch to new array's data segment.
    DEF SEG = NewSeg%
    '** Put the byte/pixel into the new array.
    POKE NewOff% + 4 + NewI% + (J% - X1%), Byte%
  NEXT J%
NEXT I%
'** Switch back to default data segment.
DEF SEG

END SUB

SUB DrawScreen
SHARED Map%(), PlayerX%, PlayerY%

'** This draws the entire "viewport".  Currently only used at the very
'** beginning of the program.  Could be used in cases where the entire
'** screen changes, such as when the player moves to a completely new
'** position, or when a new map (and/or a new tile set) is loaded.

'** "Viewport" is 13 tiles wide (0-12).
FOR I% = 0 TO 12
  '** Each tile is 24 pixels wide.
  I2% = I% * 24 + 4
  '** "Viewport" is 7 tiles high (0-6).
  FOR J% = 0 TO 6
    '** Each tile is 24 pixels high.
    J2% = J% * 24 + 4
    '** Find current position in "viewport" relative to player's
    '** position on map.
    K% = PlayerX% - 6 + I%
    L% = PlayerY% - 3 + J%
    '** Correct values if they wrap-around the map.
    IF K% < 0 THEN K% = 16 + K%
    IF L% < 0 THEN L% = 16 + L%
    IF K% > 15 THEN K% = K% - 16
    IF L% > 15 THEN L% = L% - 16
    '** I2% and J2% hold the X and Y coordinates of the upper left corner
    '** of the tile position on the screen, so we draw the entire tile
    '** offset from that position.
    EMSPUT I2%, J2%, Map%(K, L), "PSET"
  NEXT J%
NEXT I%

END SUB

SUB EMSGET (X1%, Y1%, X2%, Y2%)
'** Yes, we really need to share all these...
SHARED EMSHandle%, EMSSegment%
SHARED MaxImages%, CurrentImage%, ImageIndex() AS EMSImageIndex
SHARED MaxPages%, CurrentPage%, SpaceLeft%

'** Too many images?
IF CurrentImage% > MaxImages% THEN
  SCREEN 0
  BEEP
  PRINT "Image array limit exceeded."
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF
'** Increment our EMS pointers.  First time through, this sets our
'** current page to 1, and our space left in the page to 16384.
'** Successive times through, it calculates the current page and
'** space remaining following the last execution.
DO WHILE SpaceLeft% < 1
  CurrentPage% = CurrentPage% + 1
  SpaceLeft% = SpaceLeft% + 16384
LOOP
'** How big is this image going to be?
Size% = ((X2% - X1% + 1) * (Y2% - Y1% + 1) + 4) \ 2
'** Not enough memory left?
IF Size% * 2 - SpaceLeft% > (MaxPages% - CurrentPage% - 1) * 16384& THEN
  SCREEN 0
  BEEP
  PRINT "Not enough EMS to perform operation."
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF
'** We're okay, so GET it!
DIM GraphicArray%(Size%)
GET (X1%, Y1%)-(X2%, Y2%), GraphicArray%
'** Find out some info to pass to MemCopy.
ImageSeg% = VARSEG(GraphicArray%(0))
ImageOff% = VARPTR(GraphicArray%(0))
Start% = 16384 - SpaceLeft%
'** Switch to the appropriate page in EMS.
MapEMS EMSHandle%, CurrentPage%
'** Store the image in EMS.
MemCopy ImageSeg%, ImageOff%, EMSSegment%, Start%, Size% * 2
'** Remember where we saved this image, put the address in our index.
ImageIndex(CurrentImage%).ImagePage = CurrentPage%
ImageIndex(CurrentImage%).ImageAddress = Start%
ImageIndex(CurrentImage%).ImageSize = Size%
'** More counters to increment/decrement.
CurrentImage% = CurrentImage% + 1
SpaceLeft% = SpaceLeft% - Size% * 2

END SUB

DEFSNG A-Z
'************* EMSPages%() ****************
'*** When func% is 0, returns the total ***
'*** number of 16k pages, when func% is ***
'*** 1, returns the number of available ***
'*** 16k pages.                         ***
'******************************************
FUNCTION EMSPages% (func%)

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(180)
asm$ = asm$ + CHR$(66) + CHR$(205) + CHR$(103) + CHR$(139) + CHR$(126)
asm$ = asm$ + CHR$(6) + CHR$(137) + CHR$(29) + CHR$(139) + CHR$(126)
asm$ = asm$ + CHR$(8) + CHR$(137) + CHR$(21) + CHR$(93) + CHR$(203)

TotalPages% = 0: AvailablePages% = 0

DEF SEG = VARSEG(asm$)
   CALL Absolute(TotalPages%, AvailablePages%, SADD(asm$))
DEF SEG

IF func% = 0 THEN
   EMSPages% = TotalPages%
ELSE
   EMSPages% = AvailablePages%
END IF

END FUNCTION

DEFINT A-Z
SUB EMSPULL (Array%(), ImageNum%)
'** Yes, we really need to share all these...
SHARED EMSHandle%, EMSSegment%
SHARED MaxImages%, CurrentImage%, ImageIndex() AS EMSImageIndex

'** Image outside of array limit?
IF ImageNum% > MaxImages% THEN
  SCREEN 0
  BEEP
  PRINT "Image array limit exceeded."
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF
'** Image not defined yet?
IF ImageNum% > CurrentImage% - 1 THEN
  SCREEN 0
  BEEP
  PRINT "Undefined image number."
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF

'** Check size of array.
Size% = ImageIndex(ImageNum%).ImageSize
'** Passed array not large enough to hold image?
IF Size% > UBOUND(Array%) * 2 THEN
  SCREEN 0
  BEEP
  PRINT "Temporary array DIMed to small."
  PRINT "DIMed as:"; UBOUND(Array%)
  PRINT "Required:"; (Size% + 1) \ 2
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF
'** Find the address of the array for MemCopy.
ImageSeg% = VARSEG(Array%(0))
ImageOff% = VARPTR(Array%(0))
'** Find the address of the image for MemCopy.
Start% = ImageIndex(ImageNum%).ImageAddress
'** Switch to the appropriate page in EMS.
MapEMS EMSHandle%, ImageIndex(ImageNum%).ImagePage
'** Move the image from EMS into the array.
MemCopy EMSSegment%, Start%, ImageSeg%, ImageOff%, Size% * 2

END SUB

SUB EMSPUT (X%, Y%, ImageNum%, Mode$)
'** Yes, we really need to share all these...
SHARED EMSHandle%, EMSSegment%
SHARED MaxImages%, CurrentImage%, ImageIndex() AS EMSImageIndex

'** Image outside of array limit?
IF ImageNum% > MaxImages% THEN
  SCREEN 0
  BEEP
  PRINT "Image array limit exceeded."
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF
'** Image not defined yet?
IF ImageNum% > CurrentImage% - 1 THEN
  SCREEN 0
  BEEP
  PRINT "Undefined image number."
  '** Release the EMS memory.
  ReleaseEMS EMSHandle%
  END
END IF
'** Create an array to hold the image.
Size% = ImageIndex(ImageNum%).ImageSize
DIM GraphicArray%(Size%)
'** Find the address of the array for MemCopy.
ImageSeg% = VARSEG(GraphicArray%(0))
ImageOff% = VARPTR(GraphicArray%(0))
'** Find the address of the image for MemCopy.
Start% = ImageIndex(ImageNum%).ImageAddress
'** Switch to the appropriate page in EMS.
MapEMS EMSHandle%, ImageIndex(ImageNum%).ImagePage
'** Move the image from EMS into the array.
MemCopy EMSSegment%, Start%, ImageSeg%, ImageOff%, Size% * 2
'** Now, how do we want to put it on the screen?
SELECT CASE Mode$
  CASE "PSET"
    PUT (X%, Y%), GraphicArray%, PSET
  CASE "PRESET"
    PUT (X%, Y%), GraphicArray%, PRESET
  CASE "AND"
    PUT (X%, Y%), GraphicArray%, AND
  CASE "OR"
    PUT (X%, Y%), GraphicArray%, OR
  CASE ELSE
    PUT (X%, Y%), GraphicArray%, XOR
END SELECT

END SUB

DEFSNG A-Z
'**************** EMSstatus%() ******************
'*** Returns whether EMS is available.  -1 is ***
'*** returned if it is available, 0 otherwise ***
'************************************************
FUNCTION EMSStatus%

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(180)
asm$ = asm$ + CHR$(64) + CHR$(205) + CHR$(103) + CHR$(176) + CHR$(0)
asm$ = asm$ + CHR$(139) + CHR$(94) + CHR$(6) + CHR$(137) + CHR$(7)
asm$ = asm$ + CHR$(93) + CHR$(203)

EMS% = -1
DEF SEG = VARSEG(asm$)
   CALL Absolute(EMS%, SADD(asm$))
DEF SEG

IF EMS% = 0 THEN
   EMSStatus = -1         'EMS installed, set to BASIC's TRUE value.
ELSE
   EMSStatus = 0          'EMS not installed, set to FALSE.
END IF

END FUNCTION

DEFINT A-Z
FUNCTION fGetKey%
'** Waits until a key is pressed, then returns it in fGetKey%.

DO
  A$ = INKEY$
  '** Check palette rotation timer.
  RotTimer
LOOP UNTIL LEN(A$)
IF LEN(A$) > 1 THEN fGetKey% = -ASC(RIGHT$(A$, 1)) ELSE fGetKey% = ASC(A$)

END FUNCTION

DEFSNG A-Z
'********************** GetEMS%() ********************
'*** Function returns the handle value for a block ***
'*** of EMS memory that consists of numpages% 16k  ***
'*** pages.  You _must_ keep the handle value for  ***
'*** later calls that require the handle.  Example:***
'***                                               ***
'*** EmsHandle% = GetEMS%(5)                       ***
'***                                               ***
'*** EmsHandle% holds the handle info for a block  ***
'*** of memory 5 16k pages in size, or 80k.        ***
'*****************************************************
FUNCTION GetEMS% (numpages%)

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(139)
asm$ = asm$ + CHR$(94) + CHR$(8) + CHR$(180) + CHR$(67) + CHR$(205)
asm$ = asm$ + CHR$(103) + CHR$(139) + CHR$(94) + CHR$(6) + CHR$(137)
asm$ = asm$ + CHR$(23) + CHR$(93) + CHR$(203)

Handle% = 0
DEF SEG = VARSEG(asm$)
   CALL Absolute(BYVAL numpages%, Handle%, SADD(asm$))
DEF SEG

GetEMS% = Handle%

END FUNCTION

DEFINT A-Z
SUB InsertArray (SourceArray%(), TargetArray%(), X%, Y%)

'** First byte of array (0) holds width information,
'** multiplied by 8.
SourceWidth% = SourceArray%(0) \ 8
TargetWidth% = TargetArray%(0) \ 8
'** Find height of source array.
SourceHeight% = SourceArray%(1)
'** Find data segments and offsets for the two arrays.
SourceSeg% = VARSEG(SourceArray%(0))
SourceOff% = VARPTR(SourceArray%(0))
TargetSeg% = VARSEG(TargetArray%(0))
TargetOff% = VARPTR(TargetArray%(0))
'** Outer loop steps through height.
FOR I% = 0 TO SourceHeight% - 1
  '** Next two statements reduce math operations performed
  '** inside inner loop.
  SourceI% = I% * SourceWidth%
  TargetI% = (I% + Y%) * TargetWidth%
  '** Inner loop steps through width.
  FOR J% = 0 TO SourceWidth% - 1
    '** Switch to source array's data segment.
    DEF SEG = SourceSeg%
    '** Pull a single byte/pixel from source array.
    Byte% = PEEK(SourceOff% + 4 + SourceI% + J%)
    '** Skip color 0 for transparency effects.
    IF Byte% THEN
      '** Switch to target array's data segment.
      DEF SEG = TargetSeg%
      '** Put the byte/pixel into the new array.
      POKE TargetOff% + 4 + TargetI% + (J% + X%), Byte%
    END IF
  NEXT J%
NEXT I%
'** Switch back to default data segment.
DEF SEG

END SUB

SUB KillPalette

FOR I% = 0 TO 255
  OUT &H3C8, I%
  OUT &H3C9, 0
  OUT &H3C9, 0
  OUT &H3C9, 0
NEXT I%

END SUB

DEFSNG A-Z
'***************** MapEMS () ***********************************
'*** Sets the page of a memory block (identified by Handle%) ***
'*** that is located at the beginning of the page frame.     ***
'*** Example:                                                ***
'***                                                         ***
'*** EmsHandle% = GetEMS%(8)                                 ***
'*** MapEMS EmsHandle%, 4                                    ***
'***                                                         ***
'*** When the page frame segment is next written to, the info***
'*** will be placed starting at the 4th page in the block of ***
'*** memory represented by EmsHandle%.  This could be use,   ***
'*** for instance, to store multiple SCREEN 13 images in one ***
'*** EMS block, by moving the first 64k image into the first ***
'*** 4 16k pages (16000 * 4 = 64000) by using:               ***
'***                                                         ***
'*** MapEMS EmsHandle%, 0                                    ***
'***                                                         ***
'*** And then putting the next 64k image into the next 4 EMS ***
'*** pages by using:                                         ***
'***                                                         ***
'*** MapEMS EmsHandle%, 4                                    ***
'***                                                         ***
'*** ... and then moving the image into the memory block.    ***
'***************************************************************
SUB MapEMS (Handle%, pageoffset%)

numpages% = 4

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(139)
asm$ = asm$ + CHR$(86) + CHR$(8) + CHR$(139) + CHR$(126) + CHR$(6)
asm$ = asm$ + CHR$(139) + CHR$(118) + CHR$(10) + CHR$(177) + CHR$(0)
asm$ = asm$ + CHR$(48) + CHR$(237) + CHR$(180) + CHR$(68) + CHR$(136)
asm$ = asm$ + CHR$(200) + CHR$(137) + CHR$(243) + CHR$(205) + CHR$(103)
asm$ = asm$ + CHR$(70) + CHR$(65) + CHR$(57) + CHR$(249) + CHR$(117)
asm$ = asm$ + CHR$(242) + CHR$(93) + CHR$(203)

DEF SEG = VARSEG(asm$)
   CALL Absolute(BYVAL pageoffset%, BYVAL Handle%, BYVAL numpages%, SADD(asm$))
DEF SEG

END SUB

DEFINT A-Z
'******************************* MemCopy() ***********************
'*** Copies the number of bytes specified in 'bytes' from the  ***
'*** memory location fromseg:fromoff to the memory location    ***
'*** toseg:tooff.  To copy more than 32,767 bytes (max. of     ***
'*** 65,536 bytes) put the 'bytes' value in HEX form.          ***
'*****************************************************************
SUB MemCopy (fromseg, fromoff, toseg, tooff, bytes)

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(30)
asm$ = asm$ + CHR$(139) + CHR$(70) + CHR$(10) + CHR$(142) + CHR$(192)
asm$ = asm$ + CHR$(139) + CHR$(70) + CHR$(14) + CHR$(142) + CHR$(216)
asm$ = asm$ + CHR$(139) + CHR$(118) + CHR$(8) + CHR$(139) + CHR$(126)
asm$ = asm$ + CHR$(12) + CHR$(139) + CHR$(78) + CHR$(6) + CHR$(243)
asm$ = asm$ + CHR$(164) + CHR$(31) + CHR$(93) + CHR$(203)

DEF SEG = VARSEG(asm$)
   CALL Absolute(BYVAL fromseg, BYVAL tooff, BYVAL toseg, BYVAL fromoff, BYVAL bytes, SADD(asm$))
DEF SEG

END SUB

SUB MoveScreen (Direction%, ScrollStep%)
SHARED Map%(), PlayerX%, PlayerY%, PlayerSpeed%
SHARED Character%(), SaveBuffer%()

'** Here's a temporary array for moving all but (PlayerSpeed%)
'** worth of lines of the "viewport".  This saves us from having
'** to redraw the entire "viewport" each time we scroll a step,
'** now we only have to draw (PlayerSpeed%) lines of pixels.
DIM ScrollCap%((316& * (172 - PlayerSpeed%) + 4) \ 2)
'** Need a temporary array for holding tiles and another for
'** holding 'clipped' tiles.
DIM TileBuffer%(290), ClipBuffer%(((23 - ScrollStep%) * 24 + 4) \ 2)

'** Check palette rotation timer.
RotTimer

'** Which direction shall we scroll the "viewport"?
SELECT CASE Direction%
  CASE 0  '** North (scroll southward)
    '** Capture all but the bottom of the "viewport".
    GET (4, 4)-(315, 171 - PlayerSpeed%), ScrollCap%
    '** Cover up character so he doesn't get scrolled.
    InsertArray SaveBuffer%(), ScrollCap%(), 144, 72
    '** Recapture the background before redrawing character.
    ClipArray ScrollCap%(), SaveBuffer%(), 144, 72 - PlayerSpeed%, 167, 95 - PlayerSpeed%
    '** Put our character back.
    InsertArray Character%(), ScrollCap%(), 144, 72 - PlayerSpeed%
    '** Move it all down.
    PUT (4, PlayerSpeed% + 4), ScrollCap%, PSET
    '** Find current Y position in "viewport" relative to player's
    '** position on map.
    L% = PlayerY% - 4
    '** Correct value if it wraps-around the map.
    IF L% < 0 THEN L% = 16 + L%
    '** We're scrolling from the bottom of the tile up, the next two
    '** lines just reduce the amount of math performed, rather than
    '** having the statements performed in the loop below.
    S1% = 24 - ScrollStep% - PlayerSpeed%
    S2% = S1% + PlayerSpeed% - 1
    '** 13 tiles across.
    FOR I% = 0 TO 12
      '** Find current X position in "viewport" relative to player's
      '** position on map.
      K% = PlayerX% - 6 + I%
      '** Correct value if it wraps-around the map.
      IF K% < 0 THEN K% = 16 + K%
      IF K% > 15 THEN K% = K% - 16
      '** Pull the tile from EMS into TileBuffer%().
      EMSPULL TileBuffer%(), Map%(K%, L%)
      '** Clip it to fit.
      ClipArray TileBuffer%(), ClipBuffer%(), 0, S1%, 23, S2%
      '** PUT the clipped tile.
      PUT (I% * 24 + 4, 4), ClipBuffer%, PSET
    NEXT I%
  CASE 1  '** South (scroll northward)
    '** Capture all but the top of the "viewport".
    GET (4, PlayerSpeed% + 4)-(315, 171), ScrollCap%
    '** Cover up character so he doesn't get scrolled.
    InsertArray SaveBuffer%(), ScrollCap%(), 144, 72 - PlayerSpeed%
    '** Recapture the background before redrawing character.
    ClipArray ScrollCap%(), SaveBuffer%(), 144, 72, 167, 95
    '** Put our character back.
    InsertArray Character%(), ScrollCap%(), 144, 72
    '** Move it all up.
    PUT (4, 4), ScrollCap%, PSET
    '** Find current Y position in "viewport" relative to player's
    '** position on map.
    L% = PlayerY% + 4
    '** Correct value if it wraps-around the map.
    IF L% > 15 THEN L% = L% - 16
    '** Next statement reduces math in loop below.
    S1% = ScrollStep% + PlayerSpeed% - 1
    '** 13 tiles across.
    FOR I% = 0 TO 12
      '** Find current X position in "viewport" relative to player's
      '** position on map.
      K% = PlayerX% - 6 + I%
      '** Correct value if it wraps-around the map.
      IF K% < 0 THEN K% = 16 + K%
      IF K% > 15 THEN K% = K% - 16
      '** Pull the tile from EMS into TileBuffer%().
      EMSPULL TileBuffer%(), Map%(K%, L%)
      '** Clip it to fit.
      ClipArray TileBuffer%(), ClipBuffer%(), 0, ScrollStep%, 23, S1%
      '** PUT the clipped tile.
      PUT (I% * 24 + 4, 172 - PlayerSpeed%), ClipBuffer%, PSET
    NEXT I%
  CASE 2  '** East (scroll westward)
    '** Capture all but the leftmost of the "viewport".
    GET (PlayerSpeed% + 4, 4)-(315, 171), ScrollCap%
    '** Cover up character so he doesn't get scrolled.
    InsertArray SaveBuffer%(), ScrollCap%(), 144 - PlayerSpeed%, 72
    '** Recapture the background before redrawing character.
    ClipArray ScrollCap%(), SaveBuffer%(), 144, 72, 167, 95
    '** Put our character back.
    InsertArray Character%(), ScrollCap%(), 144, 72
    '** Move it all left.
    PUT (4, 4), ScrollCap%, PSET
    '** Find current X position in "viewport" relative to player's
    '** position on map.
    K% = PlayerX% + 7
    '** Correct value if it wraps-around the map.
    IF K% > 15 THEN K% = K% - 16
    '** Next statement reduces math in loop below.
    S1% = ScrollStep% + PlayerSpeed% - 1
    '** 7 tiles down.
    FOR I% = 0 TO 6
      '** Find current Y position in "viewport" relative to player's
      '** position on map.
      L% = PlayerY% - 3 + I%
      '** Correct value if it wraps-around the map.
      IF L% < 0 THEN L% = 16 + L%
      IF L% > 15 THEN L% = L% - 16
      '** Pull the tile from EMS into TileBuffer%().
      EMSPULL TileBuffer%(), Map%(K%, L%)
      '** Clip it to fit.
      ClipArray TileBuffer%(), ClipBuffer%(), ScrollStep%, 0, S1%, 23
      '** PUT the clipped tile.
      PUT (316 - PlayerSpeed%, I * 24 + 4), ClipBuffer%, PSET
    NEXT I%
  CASE 3  '** West (scroll eastward)
    '** Capture all but the rightmost of the "viewport".
    GET (4, 4)-(315 - PlayerSpeed%, 171), ScrollCap%
    '** Cover up character so he doesn't get scrolled.
    InsertArray SaveBuffer%(), ScrollCap%(), 144, 72
    '** Recapture the background before redrawing character.
    ClipArray ScrollCap%(), SaveBuffer%(), 144 - PlayerSpeed%, 72, 167 - PlayerSpeed%, 95
    '** Put our character back.
    InsertArray Character%(), ScrollCap%(), 144 - PlayerSpeed%, 72
    '** Move it all right.
    PUT (PlayerSpeed% + 4, 4), ScrollCap%, PSET
    '** Find current X position in "viewport" relative to player's
    '** position on map.
    K% = PlayerX% - 7
    '** Correct value if it wraps-around the map.
    IF K% < 0 THEN K% = 16 + K%
    '** We're scrolling from the right of the tile left, the next two
    '** lines just reduce the amount of math performed, rather than
    '** having the statements performed in the loop below.
    S1% = 24 - ScrollStep% - PlayerSpeed%
    S2% = S1% + PlayerSpeed% - 1
    '** 7 tiles down.
    FOR I% = 0 TO 6
      '** Find current Y position in "viewport" relative to player's
      '** position on map.
      L% = PlayerY% - 3 + I%
      '** Correct value if it wraps-around the map.
      IF L% < 0 THEN L% = 16 + L%
      IF L% > 15 THEN L% = L% - 16
      '** Pull the tile from EMS into TileBuffer%().
      EMSPULL TileBuffer%(), Map%(K%, L%)
      '** Clip it to fit.
      ClipArray TileBuffer%(), ClipBuffer%(), S1%, 0, S2%, 23
      '** PUT the clipped tile.
      PUT (4, I% * 24 + 4), ClipBuffer%, PSET
    NEXT I%
END SELECT

END SUB

DEFSNG A-Z
'****************************** NumEMSHandles%() *********************
'*** Returns the number of EMS handles presently being used (there ***
'*** are a maximum of 256 handles possible at any given time).     ***
'*********************************************************************
FUNCTION NumEMSHandles%

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(180)
asm$ = asm$ + CHR$(75) + CHR$(205) + CHR$(103) + CHR$(139) + CHR$(126)
asm$ = asm$ + CHR$(6) + CHR$(137) + CHR$(29) + CHR$(93) + CHR$(203)

NumHandles% = 0
DEF SEG = VARSEG(asm$)
   CALL Absolute(NumHandles%, SADD(asm$))
DEF SEG

NumEMSHandles% = NumHandles%

END FUNCTION

'******************************* PageFrame% ***************************
'*** Returns the segment that you will need to write to in order to ***
'*** store your data into EMS memory.  For example, PageFrame% may  ***
'*** return D000 (HEX, -12288 decimal), and then you might do this: ***
'***                                                                ***
'*** DEF SEG = PageFrame%        'D000                              ***
'*** MyData$ = "This is a block of data I want to store in EMS."    ***
'*** FOR X = 1 TO LEN(MyData$)                                      ***
'***   POKE X, ASC(MID$(MyData$, X, 1))                             ***
'*** NEXT X                                                         ***
'*** DEF SEG                                                        ***
'***                                                                ***
'*** Note, though, that you have to have a block of EMS opened with ***
'*** GetEMS%() and maped with MapEMS before you can write to the    ***
'*** block.                                                         ***
'**********************************************************************
FUNCTION PageFrame%

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(180)
asm$ = asm$ + CHR$(65) + CHR$(205) + CHR$(103) + CHR$(139) + CHR$(126)
asm$ = asm$ + CHR$(6) + CHR$(137) + CHR$(29) + CHR$(93) + CHR$(203)

PageFrameAddr% = 0
DEF SEG = VARSEG(asm$)
   CALL Absolute(PageFrameAddr%, SADD(asm$))
DEF SEG

PageFrame% = PageFrameAddr%

END FUNCTION

'****************************** ReleaseEMS() **************************
'*** Releases the EMS memory associated with Handle%.  This is very ***
'*** important to do before you exit your program, otherwise the    ***
'*** memory being used by your open handles will not be available   ***
'*** again until you reboot.                                        ***
'**********************************************************************
SUB ReleaseEMS (Handle%)

asm$ = ""
asm$ = asm$ + CHR$(85) + CHR$(137) + CHR$(229) + CHR$(180)
asm$ = asm$ + CHR$(69) + CHR$(139) + CHR$(86) + CHR$(6) + CHR$(205)
asm$ = asm$ + CHR$(103) + CHR$(93) + CHR$(203)

DEF SEG = VARSEG(asm$)
   CALL Absolute(BYVAL Handle%, SADD(asm$))
DEF SEG

END SUB

DEFINT A-Z
SUB RotatePalette (BeginColour%, EndColour%)

IF BeginColour% > EndColour% THEN
  Cstep% = -1
ELSEIF BeginColour% < EndColour% THEN
  Cstep% = 1
ELSE
  EXIT SUB
END IF
OUT &H3C7, EndColour%
RedSave% = INP(&H3C9)
GreenSave% = INP(&H3C9)
BlueSave% = INP(&H3C9)
FOR I% = BeginColour% TO EndColour% STEP Cstep%
  OUT &H3C7, I%
  Red% = INP(&H3C9)
  Green% = INP(&H3C9)
  Blue% = INP(&H3C9)
  OUT &H3C8, I%
  OUT &H3C9, RedSave%
  OUT &H3C9, GreenSave%
  OUT &H3C9, BlueSave%
  RedSave% = Red%
  GreenSave% = Green%
  BlueSave% = Blue%
NEXT I%

END SUB

SUB RotTimer
SHARED Time!

'** Time to rotate?
IF Time! + .1 < TIMER OR TIMER < Time! THEN
  '** Rotate colors in water tiles.
  Time! = TIMER
  RotatePalette 16, 23
  'BEEP
END IF

END SUB

SUB SpriteToScreen (Array%(), X%, Y%)
DIM TempBuffer%(290)

'** Special SUB only used to draw single transparent sprites.
'** Currently only used to draw character at beginning.
GET (X%, Y%)-(X% + 23, Y% + 23), TempBuffer%
InsertArray Array%(), TempBuffer%(), 0, 0
PUT (X%, Y%), TempBuffer%, PSET

END SUB

