'
'  **************************************************
'  *                                                *
'  *                 SHELSORT.BAS                   *
'  *                                                *
'  *    Shell-Metzner Sort demonstration program    *
'  *                                                *
'  *              Judson D. McClendon               *
'  *              Sun Valley Systems                *
'  *              4522 Shadow Ridge Pkwy            *
'  *              Pinson, AL 35126-2192             *
'  *                 205-680-0460                   *
'  *                                                *
'  **************************************************
'
DECLARE SUB Sort ()

CONST Max = 20000
DIM SHARED A(Max) AS INTEGER, N AS INTEGER
DIM SHARED I AS INTEGER, J AS INTEGER, L AS INTEGER, M AS INTEGER

CLS
PRINT "Shell-Metzner Sort Program"
DO
   INPUT "Array size: ", N
LOOP WHILE (N < 2 OR N > Max)
PRINT

RANDOMIZE TIMER
FOR I = 1 TO N
   A(I) = INT(RND * Max)
   PRINT A(I);
NEXT
PRINT : PRINT

T = TIMER
CALL Sort
T = TIMER - T

FOR I = 1 TO N
   PRINT A(I);
NEXT
PRINT : PRINT

PRINT USING "##,### items in ####.## seconds."; N; T
END

'                Explaination of M in Sort below
'
' From The Art of Computer Programming by Donald E. Knuth on Shell Sort:
'
'   ... the evidence indicates that it is reasonable to choose the
'   increments in the following way:
' 
'   Let M(1) = 1, M(s+1) = 3*M(s)+1, and stop with M(t) when M(t+2) >= M
' 
' Carrying Knuth's formula to N=30,000 (the maximum array size in the demo
' program) you get this series:
' 
'   M = 1, 4, 13, 40, 121, 364, 1093, 3280, 9841, 29524, 88573
'                                           ^^^^
'                                           M(t)         M(t+2)
' 
' Since you would have to calculate M(t) for each varying value of N, as a
' compromise because N will often be less than 30,000, I chose M(t-1), or
' 3280.  You can see that the expression in the program is to produce in
' reverse order the terms of M(t), M(t-1), ..., M(1).
'
SUB Sort
   M = 3280    ' See notes above
   WHILE M
      FOR J = 1 TO N - M
         L = J + M
         B = A(L)
         FOR I = J TO 1 STEP -M
            IF A(I) > B THEN
               A(I + M) = A(I)
               L = I
            ELSE
               I = 1
            END IF
         NEXT
         A(L) = B
      NEXT
      M = (M - 1) \ 3   ' See notes above
   WEND
END SUB
